/*
  PARA TODOS LOS PROCEDIMIENTOS EXISTIR UN PARAMETRO DE SALIDA RSP, QUE DEVOLVERA OK SI EL PROCEDIMIENTO
  SE HA REALIZADO CORRECTAMENTE Y UN MENSAJE DE ERROR EN CASO CONTRARIO.
*/


/****************************************************************************************************
*	
*		PROCEDIMIENTOS DE LA BASE DE DATOS
*
****************************************************************************************************/


/*-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_insertarlog
--  OBJETO: Insertar en la tabla logs las llamadas a los procedimientos
--  PARAMETROS DE ENTRADA:  nombre_proc, fecha, paramentrada
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------*/

CREATE OR REPLACE PROCEDURE p_insertaLog (
	nombre_proc IN logs.nom%TYPE,
	fecha IN logs.fecha%TYPE,
	paramentrada IN logs.param_entrada%TYPE,
	RSP IN Varchar2
) IS

  BEGIN
	INSERT INTO LOGS (nom, fecha, param_entrada, param_salida)
	VALUES (nombre_proc, fecha, paramentrada,RSP);
	COMMIT;	

  EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		DBMS_OUTPUT.PUT_LINE('ERROR: Error al almacenar el log.');
	
	WHEN OTHERS THEN
	
		DBMS_OUTPUT.PUT_LINE('ERROR: Error al insertar el log: ' || sqlerrm);
	
END;
/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altaestado
--  OBJETO: Dar de alta los distintos estados de las elecciones
--  PARAMETROS DE ENTRADA:  nombre, descripcion
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altaestado (
	nom IN estado.nombre%TYPE,
	des in estado.descripcion%TYPE,
	RSP OUT Varchar2
) IS

  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;

  parametros varchar2(250);
  nombre estado.nombre%TYPE;

  BEGIN
	
	parametros := nom ||','||des;

	IF (nom IS NULL OR nom = '') THEN
		 RAISE obligatorio;
	END IF;

	nombre:=lower(nom);

	IF(nombre <> 'edicion' and nombre <> 'pendiente' and nombre<> 'abierta' and nombre <> 'finalizada') THEN
		RAISE valorincorrecto;
	END IF;
 
	

	INSERT INTO ESTADO(nombre,descripcion)
	VALUES(nombre,des);
	

	RSP := 'OK, se ha dado de alta el estado correctamente';
	p_insertaLog('p_altaestado',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El estado de la votacion ya existe';
		ROLLBACK;
		p_insertaLog('p_altaestado',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el estado de la votacion';
		ROLLBACK;
		p_insertaLog('p_altaestado',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altaestado',SYSDATE, parametros, RSP);
		commit;
	
	WHEN valorincorrecto THEN
	
		RSP := 'ERROR: estado no valido, solo se puede dar de alta los estados edicion, pendiente, abierta, finalizada';
		ROLLBACK;
		p_insertaLog('p_altaestado',SYSDATE, parametros, RSP);
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el estado de la votacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altaestado',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altambito
--  OBJETO: Dar de alta los distintos ambitos de las elecciones
--  PARAMETROS DE ENTRADA:  nombre, descripcion
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altambito (
	des in ambito.descripcion%TYPE,
	RSP OUT Varchar2
) IS

  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;

  parametros varchar2(250);
  descripcion ambito.descripcion%TYPE;

  BEGIN
	
        descripcion:=lower(des);
	parametros := descripcion;
	
	IF (des IS NULL OR des = '') THEN
		 RAISE obligatorio;
	END IF;

	

	IF(descripcion <> 'global' and descripcion <> 'nacional' and descripcion<> 'regional' and descripcion <> 'local') THEN
		RAISE valorincorrecto;
	END IF;
 
	

	INSERT INTO AMBITO(descripcion)
	VALUES(descripcion);
	

	RSP := 'OK, se ha dado de alta el ambito de la votacion correctamente';
	p_insertaLog('p_altambito',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El ambito de la votacion ya existe';
		ROLLBACK;
		p_insertaLog('p_altambito',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el ambito de la votacion';
		ROLLBACK;
		p_insertaLog('p_altambito',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altambito',SYSDATE, parametros, RSP);
		commit;
	
	WHEN valorincorrecto THEN
	
		RSP := 'ERROR: ambito no valido, solo se puede dar de alta los estados global, nacional, regional, local';
		ROLLBACK;
		p_insertaLog('p_altambito',SYSDATE, parametros, RSP);
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el ambito de la votacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altambito',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacensoglobal
--  OBJETO: Dar de alta el censo global de las elecciones
--  PARAMETROS DE ENTRADA:  
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altacensoglobal (
	RSP OUT Varchar2
) IS

  parametros varchar2(250);

  BEGIN
	parametros:=NULL;	
 
	INSERT INTO CENSO_GLOBAL(NOMBRE)
	VALUES('EUROPA');
        
	RSP := 'OK, se ha dado de alta el censo global de la votacion correctamente';

	p_insertaLog('p_altacensoglobal',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El censo global ya existe';
		ROLLBACK;
		p_insertaLog('p_altacensoglobal',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el censo global';
		ROLLBACK;
		p_insertaLog('p_altacensoglobal',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el censo global: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacensoglobal',SYSDATE, parametros, RSP);
		commit;
	
END;
/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacensonacional
--  OBJETO: Dar de alta los censos nacionales de las elecciones
--  PARAMETROS DE ENTRADA: codigo
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altacensonacional (
        codigo in CENSO_NACIONAL.COD_PAIS%TYPE,
        RSP OUT Varchar2
) IS

  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  PRAGMA EXCEPTION_INIT(valorincorrecto,-02290);
  
  cod CENSO_NACIONAL.COD_PAIS%TYPE;
  parametros varchar2(250);

  BEGIN
	
	IF (codigo IS NULL OR codigo = '') THEN
		 RAISE obligatorio;
	END IF;
  
        cod:=upper(codigo);
        
 
	parametros := cod;

-- Cod_cglobal=1 corresponde al valor de Europa
	INSERT INTO CENSO_NACIONAL(cod_cglobal,cod_pais,nom_pais)
	VALUES(1,cod,DECODE(cod,'AT','AUSTRIA',
                              'BE','BELGICA',
                              'BG','BULGARIA',
                              'CY','CHIPRE',
                              'CZ','REPUBLICA CHECA',
                              'DK','DINAMARCA',
                              'EE','ESTONIA',
                              'FI','FINLANDIA',
                              'FR','FRANCIA',
                              'DE','ALEMANIA',
                              'GR','GRECIA',
                              'HU','HUNGRIA',
                              'IE','IRLANDA',
                              'IT','ITALIA',
                              'LV','LETONIA',
                              'LT','LITUANIA',
                              'LU','LUXEMBURGO',
                              'MT','MALTA',
                              'NL','PAISES BAJOS',
                              'PL','POLONIA',
                              'PT','PORTUGAL',
                              'RO','RUMANIA',
                              'SK','ESLOVAQUIA',
                              'SI','ESLOVENIA',
                              'ES','ESPAA',
                              'SE','SUECIA',
                              'GB','REINO UNIDO'
                              ));
	

	RSP := 'OK, se ha dado de alta el Censo Nacional correctamente';
	p_insertaLog('p_altacensonacional',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El Censa_Nacional ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altacensonacional',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el Censo Nacional';
		ROLLBACK;
		p_insertaLog('p_altacensonacional',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altacensonacional',SYSDATE, parametros, RSP);
		commit;
	
	WHEN valorincorrecto THEN
	
		RSP := 'ERROR: Censo Nacional no valido, solo se puede dar de alta los codigos de la norma ISO 3166-1 alfa 2';
		ROLLBACK;
		p_insertaLog('p_altacensonacional',SYSDATE, parametros, RSP);
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Censo Nacional: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacensonacional',SYSDATE, parametros, RSP);
		commit;
	
END;
/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacensoregional
--  OBJETO: Dar de alta los censos regionales de las elecciones
--  PARAMETROS DE ENTRADA: pais,codigo, nombre 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altacensoregional (
	pais in CENSO_REGIONAL.COD_PAIS%TYPE,
        codigo in CENSO_REGIONAL.COD_REGION%TYPE,
	nombre in CENSO_REGIONAL.NOM_REGION%TYPE,
        RSP OUT Varchar2
) IS

  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  
  existe number:=null;
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);  

  codpais CENSO_REGIONAL.COD_PAIS%TYPE;
  codregion CENSO_REGIONAL.COD_REGION%TYPE;

  parametros varchar2(250);

  BEGIN	
  
        codpais:=upper(pais);
	codregion:=upper(codigo);

	parametros := pais ||','||codregion ||','||nombre;

-- Compruebo que el pais ha sido dado de alta previamente
	
	SELECT COUNT(cod_cnacional) into existe FROM CENSO_NACIONAL WHERE cod_pais=codpais;

	IF (existe<1) THEN 
		RAISE valorincorrecto;
	END IF;
 

	INSERT INTO CENSO_REGIONAL(cod_pais,cod_region,nom_region)
	VALUES(codpais,codregion,nombre);
	

	RSP := 'OK, se ha dado de alta el Censo Regional correctamente';

	p_insertaLog('p_altacensoregionall',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El CensO Regional ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altacensoregional',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el Censo Regional';
		ROLLBACK;
		p_insertaLog('p_altacensoregional',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altacensoregional',SYSDATE, parametros, RSP);
		commit;

	WHEN valorincorrecto THEN
	
		RSP := 'ERROR: El pais no existe o no ha sido dado de alta en el censo nacional';
		ROLLBACK;
		p_insertaLog('p_altacensoregional',SYSDATE, parametros, RSP);
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Censo Regional: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacensoregional',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editacensoregional
--  OBJETO: modificar los censos regionales de las elecciones
--  PARAMETROS DE ENTRADA: cod_cregional,pais,region, nombre 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_editacensoregional (
	cregional in CENSO_REGIONAL.COD_CREGIONAL%TYPE,
	pais in CENSO_REGIONAL.COD_PAIS%TYPE,
        region in CENSO_REGIONAL.COD_REGION%TYPE,
	nombre in CENSO_REGIONAL.NOM_REGION%TYPE,
        RSP OUT Varchar2
) IS

  
  claveajena EXCEPTION;
  noencontrado EXCEPTION;
  obligatorio EXCEPTION;
  localidades EXCEPTION;
  electores EXCEPTION;

  PRAGMA EXCEPTION_INIT(claveajena,-02291);
  PRAGMA EXCEPTION_INIT(localidades,-02292);
  

  existe number:=null;
  

  codpais CENSO_REGIONAL.COD_PAIS%TYPE;
  codregion CENSO_REGIONAL.COD_REGION%TYPE;
  
  v_pais CENSO_REGIONAL.COD_PAIS%TYPE;
  V_region CENSO_REGIONAL.COD_REGION%TYPE;

  parametros varchar2(250);

  BEGIN	
  
        codpais:=upper(pais);
	codregion:=upper(region);

	parametros := cregional ||','||pais ||','||region ||','||nombre;

-- Compruebo que el codigo de censo regional es distinto de cero o nulo

	IF (cregional IS NULL OR cregional = 0) THEN
		 RAISE obligatorio;
	END IF;
-- Compruebo que el censo regional existe
      SELECT COUNT(cod_cregional) into existe FROM CENSO_REGIONAL WHERE cod_cregional=cregional;

	IF (existe<1) THEN 
		RAISE noencontrado;
	END IF;

-- Si hay electores asignados a dicha region no permitiremos cambiar el censo regional, ya que si lo hacemos podemos estar cambiando a los electores de pais o region
        SELECT COUNT(cod_elector) into existe FROM ELECTOR WHERE cod_cregional=cregional;

	IF (existe>0) THEN 
		RAISE electores;
	END IF;

	
-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro
	UPDATE CENSO_REGIONAL SET cod_pais =DECODE(pais,'',cod_pais,NULL,cod_pais,codpais), cod_region=DECODE(region,'',cod_region,NULL,cod_region,codregion),nom_region=DECODE(nombre,'',nom_region,NULL,nom_region,nombre)
					   WHERE cod_cregional = cregional;	 
              

	RSP := 'OK, se ha modificado el Censo Regional correctamente';

	p_insertaLog('p_editacensoregional',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION

	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El CensO Regional ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el Censo Regional';
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);
		commit;

	WHEN noencontrado THEN
	 	RSP := 'ERROR: El Censo Regional no existe';
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);	

	WHEN claveajena THEN
	 	RSP := 'ERROR: El pais no existe o no ha sido dado de alta en el censo Nacional';
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);	

	WHEN localidades THEN
	 	RSP := 'ERROR: No se puede cambiar el censo regional mientras haya localidades asignadas a esa region';
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);	

	WHEN electores THEN
	 	RSP := 'ERROR: No se puede cambiar el censo regional mientras haya electores asignados a dicha region';
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar el Censo Regional: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);
		commit;
	
END;


/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajacensoregional
--  OBJETO: Dar de baja un censo electoral regional
--  PARAMETROS DE ENTRADA:v_cregion
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- No se puede dar de baja un censo regional mientras tenga asignado algun elector en el censo regional o cualquiera de
-- sus censos locales dependientes
-- o este asignado a una votacion en estado de edicion o pendiente.

create or replace PROCEDURE p_bajacensoregional (
 	v_cregion in CENSO_REGIONAL.COD_CREGIONAL%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;
electores EXCEPTION;


PRAGMA EXCEPTION_INIT(electores,-02292);


aux number(1);
v_pais CENSO_REGIONAL.COD_PAIS%TYPE;
v_nacional CENSO_NACIONAL.COD_CNACIONAL%TYPE;

parametros VARCHAR2(500);


BEGIN

 	parametros := v_cregion;
	
	IF ((v_cregion IS NULL OR v_cregion = 0)) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que el censo electoral existe
       select count(cod_cregional) into aux from censo_regional where cod_cregional=v_cregion;
       if (aux<1) then
          raise noencontrado;
       end if;

-- obtenemos el pais al que pertenece el censo
	select cod_pais into v_pais from censo_regional where cod_cregional=v_cregion;
-- obtenemos el codigo de censo nacional correspondiente a ese pais
        select cod_cnacional into v_nacional from censo_nacional where cod_pais=v_pais;
-- comprobamos que no hay ninguna votacion en estado de edicion o pendiente que tenga asociado el censo regional o el censo nacional al que pertenece
	select count(censos_habilitados.cod_votacion) into aux from votacion, censos_habilitados where votacion.cod_votacion=censos_habilitados.cod_votacion and (votacion.estado=1 or votacion.estado=2) and (censos_habilitados.cod_censo=v_cregion or censos_habilitados.cod_censo=v_nacional);
	if (aux>0) then
          raise nomodificar;
        end if;

	DELETE CENSO WHERE COD_CENSO = v_cregion;

	
	RSP := 'OK, se ha dado de baja el censo electoral regional correctamente';
	p_insertaLog('p_bajacensoregional',SYSDATE,parametros, RSP);
	COMMIT;

	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: el censo electoral regional no existe';
		ROLLBACK;
		p_insertaLog('p_bajacensoregional',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajacensoregional',SYSDATE, parametros, RSP);	
		commit;

	WHEN electores THEN
	
		RSP := 'ERROR: No se puede eliminar el censo regional mientras existan electores asociados al mismo o cualquiera de sus censos locales asociados';
		ROLLBACK;
		p_insertaLog('p_bajacensoregional',SYSDATE, parametros, RSP);	
		commit;

	WHEN nomodificar THEN
	
		RSP := 'ERROR: No se puede eliminar el censo regional mientras este asociado a alguna votacion en estado de edicion o pendiente';
		ROLLBACK;
		p_insertaLog('p_bajacensoregional',SYSDATE, parametros, RSP);	
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja el censo regional: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajacensoregional',SYSDATE, parametros, RSP);	
		commit;

END;



/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacensolocal
--  OBJETO: Dar de alta los censos locales de las elecciones
--  PARAMETROS DE ENTRADA: pais,codigo, nombre 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altacensolocal (
	pais in CENSO_LOCAL.COD_PAIS%TYPE,
	region in CENSO_LOCAL.COD_REGION%TYPE,
        codigo in CENSO_LOCAL.COD_LOCALIDAD%TYPE,
	nombre in CENSO_LOCAL.NOM_LOCALIDAD%TYPE,
        RSP OUT Varchar2
) IS

  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  pertenencia EXCEPTION;  

  existe number:=null;
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);  

  codpais CENSO_LOCAL.COD_PAIS%TYPE;
  codregion CENSO_LOCAL.COD_REGION%TYPE;
  codcodigo CENSO_LOCAL.COD_LOCALIDAD%TYPE;


  parametros varchar2(250);

  BEGIN	
  
        codpais:=upper(pais);
	codregion:=upper(region);
	codcodigo:=upper(codigo);

	parametros := codpais ||','||codregion||','||codcodigo ||','||nombre;


	
	IF (codregion IS null or codregion=' ')  or (codpais IS null or codpais=' ') then
		RAISE obligatorio;
	END IF;


	-- Compruebo que el pais ha sido dado de alta previamente
	
	SELECT COUNT(cod_cnacional) into existe FROM CENSO_NACIONAL WHERE cod_pais=codpais;

	IF (existe<1) THEN 
		RAISE valorincorrecto;
	END IF;


	-- Compruebo que la region ha sido dado de alta previamente
	
	    SELECT COUNT(cod_cregional) into existe FROM CENSO_REGIONAL WHERE cod_region=codregion;

	    IF (existe<1) THEN 
		RAISE valorincorrecto;
	    END IF;



	-- Compruebo que la region pertenece al pais 
		
	   SELECT COUNT(cod_cregional) into existe FROM CENSO_REGIONAL WHERE (cod_region=codregion) and (cod_pais=codpais);
	
	   IF(existe<1) THEN
		RAISE pertenencia;
	   END IF;
	
 	

	INSERT INTO CENSO_LOCAL(cod_pais,cod_region,cod_localidad,nom_localidad)
	VALUES(codpais,codregion,codcodigo,nombre);
	

	RSP := 'OK, se ha dado de alta el Censo Local correctamente';

	p_insertaLog('p_altacensolocal',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El Censo local ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altacensolocal',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el Censo Local';
		ROLLBACK;
		p_insertaLog('p_altacensolocal',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altacensolocal',SYSDATE, parametros, RSP);
		commit;

	WHEN valorincorrecto THEN
	
		RSP := 'ERROR: El pais o la region no existe o no ha sido dado de alta en el censo regional';
		ROLLBACK;
		p_insertaLog('p_altacensolocal',SYSDATE, parametros, RSP);
		commit;
	
	WHEN pertenencia THEN
	
		RSP := 'ERROR: la region no pertenece a ese pais';
		ROLLBACK;
		p_insertaLog('p_altacensolocal',SYSDATE, parametros, RSP);
		commit;	

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Censo Local: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacensolocal',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editacensolocal
--  OBJETO: modificar los censos electorales de las elecciones
--  PARAMETROS DE ENTRADA: cod_clocal,pais,region, localidad,nombre 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

create or replace PROCEDURE p_editacensolocal (
	clocal in CENSO_LOCAL.COD_CLOCAL%TYPE,
	pais in CENSO_LOCAL.COD_PAIS%TYPE,
        region in CENSO_LOCAL.COD_REGION%TYPE,
	localidad CENSO_LOCAL.COD_LOCALIDAD%TYPE,
	nombre in CENSO_LOCAL.NOM_LOCALIDAD%TYPE,
        RSP OUT Varchar2
) IS

  
  claveajena EXCEPTION;
  noencontrado EXCEPTION;
  obligatorio EXCEPTION;
  centroelectoral EXCEPTION;
  electores EXCEPTION;

  existe number:=null;

  PRAGMA EXCEPTION_INIT(claveajena,-02291);


  

  codpais CENSO_LOCAL.COD_PAIS%TYPE;
  codregion CENSO_LOCAL.COD_REGION%TYPE;
  codlocalidad CENSO_LOCAL.COD_LOCALIDAD%TYPE;

  parametros varchar2(250);

  BEGIN	
  
        codpais:=upper(pais);
	codregion:=upper(region);
	codlocalidad :=upper(localidad);

	parametros := clocal ||','||pais ||','||region ||','||localidad || ',' || nombre;

-- Compruebo que el codigo de censo local es distinto de cero o nulo

	IF (clocal IS NULL OR clocal = 0) THEN
		 RAISE obligatorio;
	END IF;

-- Compruebo que el censo local existe
        SELECT COUNT(cod_clocal) into existe FROM CENSO_LOCAL WHERE cod_clocal=clocal;

	IF (existe<1) THEN 
		RAISE noencontrado;
	END IF;


-- Si hay electores asignados a dicha localidad no permitiremos cambiar el censo local, ya que si lo hacemos podemos estar cambiando a los electores de pais , region o localidad
        SELECT COUNT(cod_elector) into existe FROM ELECTOR WHERE cod_clocal=clocal;

	IF (existe>0) THEN 
		RAISE electores;
	END IF;

-- Si hay centros electorales asignados a dicha localidad no permitiremos cambiar el censo local
        SELECT COUNT(cod_centro) into existe FROM CENTRO_ELECTORAL WHERE cod_clocal=clocal;

	IF (existe>0) THEN 
		RAISE centroelectoral;
	END IF;

	-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro
	UPDATE CENSO_LOCAL SET cod_pais = DECODE(pais,'',cod_pais,NULL,cod_pais,codpais), cod_region=DECODE(region,'',cod_region,NULL,cod_region,codregion),cod_localidad=DECODE(localidad,'',cod_localidad,NULL,cod_localidad,codlocalidad), nom_localidad=DECODE(nombre,'',nom_localidad,NULL,nom_localidad,nombre)
					   WHERE cod_clocal = clocal;	 	


	RSP := 'OK, se ha modificado el Censo Local correctamente';

	p_insertaLog('p_editacensolocal',SYSDATE,parametros, RSP);
	
	COMMIT;


  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El CensO Local ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_editacensolocal',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el Censo Local';
		ROLLBACK;
		p_insertaLog('p_editacensolocal',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editacensolocal',SYSDATE, parametros, RSP);
		commit;

	WHEN claveajena THEN
	
		RSP := 'ERROR: El pais no existe, la region no existe, o la region no pertenece a ese pais';
		ROLLBACK;
		p_insertaLog('p_editacensolocal',SYSDATE, parametros, RSP);
		commit;
	
	WHEN electores THEN
	
		RSP := 'ERROR: No se puede modificar el censo local mientras existan electores asignados a dicho censo electoral';
		ROLLBACK;
		p_insertaLog('p_editacensolocal',SYSDATE, parametros, RSP);
		commit;
        WHEN centroelectoral THEN
	
		RSP := 'ERROR: No se puede modificar el censo local mientras existan  Centros electorales asignados a dicho censo electoral';
		ROLLBACK;
		p_insertaLog('p_editacensolocal',SYSDATE, parametros, RSP);
		commit;
                
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El Censo Local no existe';
		ROLLBACK;
		p_insertaLog('p_editacensolocal',SYSDATE, parametros, RSP);	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar el Censo Local: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editacensoregional',SYSDATE, parametros, RSP);
		commit;
	
END;
/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajacensolocal
--  OBJETO: Dar de baja un censo electoral local
--  PARAMETROS DE ENTRADA:v_clocal
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- No se puede dar de baja un censo local mientras tenga asignado algun elector o este asignado a una votacion 
-- en estado de edicion o pendiente.

create or replace PROCEDURE p_bajacensolocal (
 	v_clocal in CENSO_LOCAL.COD_CLOCAL%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;
electores EXCEPTION;


PRAGMA EXCEPTION_INIT(electores,-02292);


aux number(1);


parametros VARCHAR2(500);


BEGIN

 	parametros := v_clocal;
	
	IF ((v_clocal IS NULL OR v_clocal = 0)) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que el censo electoral existe
       select count(cod_clocal) into aux from censo_local where cod_clocal=v_clocal;
       if (aux<1) then
          raise noencontrado;
       end if;

-- comprobamos que no hay ninguna votacion en estado de edicion o pendiente que tenga asociado el censo
	select count(censos_habilitados.cod_votacion) into aux from votacion, censos_habilitados where votacion.cod_votacion=censos_habilitados.cod_votacion and (votacion.estado=1 or votacion.estado=2) and censos_habilitados.cod_censo=v_clocal;
	if (aux>0) then
          raise nomodificar;
        end if;

	DELETE CENSO WHERE COD_CENSO = v_clocal;

	
	RSP := 'OK, se ha dado de baja el censo electoral local correctamente';
	p_insertaLog('p_bajacensolocal',SYSDATE,parametros, RSP);
	COMMIT;

	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: el censo electoral local no existe';
		ROLLBACK;
		p_insertaLog('p_bajacensolocal',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajacensolocal',SYSDATE, parametros, RSP);	
		commit;

	WHEN electores THEN
	
		RSP := 'ERROR: No se puede eliminar el censo local mientras existan electores asociados al mismo';
		ROLLBACK;
		p_insertaLog('p_bajacensolocal',SYSDATE, parametros, RSP);	
		commit;

	WHEN nomodificar THEN
	
		RSP := 'ERROR: No se puede eliminar el censo local mientras este asociado a alguna votacion en estado de edicion o pendiente';
		ROLLBACK;
		p_insertaLog('p_bajacensolocal',SYSDATE, parametros, RSP);	
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja el censo local: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajacensolocal',SYSDATE, parametros, RSP);	
		commit;

END;


/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacentroelectoral
--  OBJETO: Dar de alta los distintos centros electorales
--  PARAMETROS DE ENTRADA: localidad,nombre, direccion 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altacentroelectoral (
	localidad in CENTRO_ELECTORAL.COD_CLOCAL%TYPE,
        nombre in CENTRO_ELECTORAL.NOMBRE%TYPE,
	direccion in CENTRO_ELECTORAL.DIRECCION%TYPE,
        RSP OUT Varchar2
) IS

  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  
  existe number:=null;
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);  

  codlocal CENTRO_ELECTORAL.COD_CLOCAL%TYPE;
  codnombre CENTRO_ELECTORAL.NOMBRE%TYPE;

  parametros varchar2(250);

  BEGIN	
  
        codlocal:=upper(localidad);
	codnombre:=upper(nombre);

	parametros := codlocal ||','||codnombre ||','||direccion;

-- Compruebo que la localidad ha sido dada de alta previamente
	
	SELECT COUNT(cod_clocal) into existe FROM CENSO_LOCAL WHERE cod_clocal=codlocal;

	IF (existe<1) THEN 
		RAISE valorincorrecto;
	END IF;
 

	INSERT INTO CENTRO_ELECTORAL(cod_clocal,nombre,direccion)
	VALUES(codlocal,codnombre,direccion);
	

	RSP := 'OK, se ha dado de alta el Centro Electoral correctamente';

	p_insertaLog('p_altacentroelectoral',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El Centro Electoral ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altacentroelectoral',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el Centro Electoral';
		ROLLBACK;
		p_insertaLog('p_altacentroelectoral',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altacentroelectoral',SYSDATE, parametros, RSP);
		commit;

	WHEN valorincorrecto THEN
	
		RSP := 'ERROR: La localidad no existe o no ha sido dado de alta en el censo local';
		ROLLBACK;
		p_insertaLog('p_altacentroelectoral',SYSDATE, parametros, RSP);
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Centro Electoral: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacentroelectoral',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editacentroelectoral
--  OBJETO: modificar los centros electorales donde pueden acercarse los ciudadanos a votar
--  PARAMETROS DE ENTRADA: cod_clocal,pais,region, localidad,nombre 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------


create or replace PROCEDURE p_editacentroelectoral (
	centro in CENTRO_ELECTORAL.COD_CENTRO%TYPE,
	localidad in CENTRO_ELECTORAL.COD_CLOCAL%TYPE,
        enombre in CENTRO_ELECTORAL.NOMBRE%TYPE,
	edireccion CENTRO_ELECTORAL.DIRECCION%TYPE,
        RSP OUT Varchar2
) IS

  
  claveajena EXCEPTION;
  noencontrado EXCEPTION;
  obligatorio EXCEPTION;

  existe number:=null;

  PRAGMA EXCEPTION_INIT(claveajena,-02291);
 

  parametros varchar2(250);

  BEGIN	
  
        
	parametros := centro ||','||localidad ||','||enombre ||','||edireccion;

-- Compruebo que el codigo de centro electoral es distinto de cero o nulo

	IF (centro IS NULL OR centro = 0) THEN
		 RAISE obligatorio;
	END IF;

-- Compruebo que el centro electoral existe
        SELECT COUNT(cod_centro) into existe FROM CENTRO_ELECTORAL WHERE cod_centro=centro;

	IF (existe<1) THEN 
		RAISE noencontrado;
	END IF;



	-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro
	UPDATE CENTRO_ELECTORAL SET cod_clocal = DECODE(localidad,0 ,cod_clocal,NULL,cod_clocal,localidad), nombre=DECODE(enombre,'',nombre,NULL,nombre,enombre),direccion=DECODE(edireccion,'',direccion,NULL,direccion,edireccion)
					   WHERE cod_centro = centro;	 	


	RSP := 'OK, se ha modificado el Centro electoral correctamente';

	p_insertaLog('p_editacentroelectoral',SYSDATE,parametros, RSP);
	
	COMMIT;


  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El Centro electoral ya ha sido dado de alta';
		ROLLBACK;
		p_insertaLog('p_editacentroelectoral',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el Centro electoral';
		ROLLBACK;
		p_insertaLog('p_editacentroelectoral',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editacentroelectoral',SYSDATE, parametros, RSP);
		commit;

	WHEN claveajena THEN
	
		RSP := 'ERROR: El censo local no existe';
		ROLLBACK;
		p_insertaLog('p_editacentroelectoral',SYSDATE, parametros, RSP);
		commit;	
                
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El Centro electoral no existe';
		ROLLBACK;
		p_insertaLog('p_editacentroelectoral',SYSDATE, parametros, RSP);	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar el Centro electoral: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editacentroelectoral',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajacentro
--  OBJETO: Dar de baja un centro electoral
--  PARAMETROS DE ENTRADA:v_centro
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- Si borro un centro electoral se borran las mesas asociadas a ese centro salvo que haya electores asociados a las mismas
-- en ese caso los electores se tendrian que pasar a otro centro electoral para que se pudiera eliminar. 


CREATE OR REPLACE
PROCEDURE p_bajacentro (
 	v_centro in CENTRO_ELECTORAL.COD_CENTRO%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
electores EXCEPTION;


PRAGMA EXCEPTION_INIT(electores,-02292);


aux number(1);


parametros VARCHAR2(500);


BEGIN

 	parametros := v_centro;
	
	IF ((v_centro IS NULL OR v_centro = 0)) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que el centro electoral existe
       select count(cod_centro) into aux from centro_electoral where cod_centro=v_centro;
       if (aux<1) then
          raise noencontrado;
       end if;

	DELETE CENTRO_ELECTORAL WHERE COD_CENTRO = v_centro;

	
	RSP := 'OK, se ha dado de baja el centro electoral correctamente';
	p_insertaLog('p_bajacentro',SYSDATE,parametros, RSP);
	COMMIT;

	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: el centro electoral no existe';
		ROLLBACK;
		p_insertaLog('p_bajacentro',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajacentro',SYSDATE, parametros, RSP);	
		commit;

	WHEN electores THEN
	
		RSP := 'ERROR: No se puede eliminar el centro electoral mientras existan mesas del mismo con electores asociados';
		ROLLBACK;
		p_insertaLog('p_bajacentro',SYSDATE, parametros, RSP);	
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja el centro electoral: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajacentro',SYSDATE, parametros, RSP);	
		commit;

END;

/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altamesa
--  OBJETO: Dar de alta los distintos mesas electorales
--  PARAMETROS DE ENTRADA: centro,mesa
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altamesa (
	centro in MESA.COD_CENTRO%TYPE,
        mesa in MESA.COD_MESA%TYPE,
        RSP OUT Varchar2
) IS

  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  
  existe number:=null;
 
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);  

  
  parametros varchar2(250);

  BEGIN	

	parametros := centro ||','||mesa;

-- Compruebo que el Centro Electoral ha sido dado de alta previamente
	
	SELECT COUNT(cod_centro) into existe FROM CENTRO_ELECTORAL WHERE cod_centro=centro;

	IF (existe<1) THEN 
		RAISE valorincorrecto;
	END IF;
 

	INSERT INTO MESA(cod_centro,cod_mesa)
	VALUES(centro,mesa);
	

	RSP := 'OK, se ha dado de alta la mesa Electoral correctamente';

	p_insertaLog('p_altamesa',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR:La mesa Electoral ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altamesa',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar la mesa Electoral';
		ROLLBACK;
		p_insertaLog('p_altamesa',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altamesa',SYSDATE, parametros, RSP);
		commit;

	WHEN valorincorrecto THEN
	
		RSP := 'ERROR: El centro electoral no existe o no ha sido dado de alta';
		ROLLBACK;
		p_insertaLog('p_altamesa',SYSDATE, parametros, RSP);
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta la mesa Electoral: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altamesa',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajamesa
--  OBJETO: Dar de baja una mesa electoral
--  PARAMETROS DE ENTRADA:v_centro, v_mesa
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- No se puede dar de baja una mesa electoral mientras existan electores asociados a dicha mesa

CREATE OR REPLACE
PROCEDURE p_bajamesa (
 	v_centro in MESA.COD_CENTRO%TYPE,
	v_mesa in MESA.COD_MESA%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
electores EXCEPTION;


PRAGMA EXCEPTION_INIT(electores,-02292);


aux number(1);


parametros VARCHAR2(500);


BEGIN

 	parametros := v_centro ||','||v_mesa;
	
	IF ((v_centro IS NULL OR v_centro = 0) or (v_mesa=NULL OR v_mesa='')) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que el centro electoral existe
       select count(cod_centro) into aux from centro_electoral where cod_centro=v_centro;
       if (aux<1) then
          raise noencontrado;
       end if;

	DELETE MESA WHERE COD_CENTRO = v_centro AND COD_MESA=v_mesa;

	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
        END IF;
	
	RSP := 'OK, se ha dado de baja la mesa correctamente';
	p_insertaLog('p_bajamesa',SYSDATE,parametros, RSP);
	COMMIT;

	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: la mesa electoral o el centro electoral no existe';
		ROLLBACK;
		p_insertaLog('p_bajamesa',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajamesa',SYSDATE, parametros, RSP);	
		commit;

	WHEN electores THEN
	
		RSP := 'ERROR: No se puede eliminar la mesa mientras existan electores asociados a la misma';
		ROLLBACK;
		p_insertaLog('p_bajamesa',SYSDATE, parametros, RSP);	
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja la mesa electoral: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajamesa',SYSDATE, parametros, RSP);	
		commit;

END;

/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altavotonulo
--  OBJETO: Dar de alta el objeto voto nulo
--  PARAMETROS DE ENTRADA:  
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altavotonulo (
	RSP OUT Varchar2
) IS

  parametros varchar2(250);

  BEGIN
	parametros:=NULL;	
 
	INSERT INTO VOTO_NULO(DESCRIPCION)
	VALUES('Voto Nulo');
        
	RSP := 'OK, se ha dado de alta el voto nulo correctamente';

	p_insertaLog('p_altavotonulo',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El voto nulo ya existe';
		ROLLBACK;
		p_insertaLog('p_altavotonulo',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el voto nulo';
		ROLLBACK;
		p_insertaLog('p_altacensoglobal',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el voto nulo: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altavotonulo',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altavotoblanco
--  OBJETO: Dar de alta el objeto voto en blanco
--  PARAMETROS DE ENTRADA:  
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altavotoblanco (
	RSP OUT Varchar2
) IS

  parametros varchar2(250);

  BEGIN
	parametros:=NULL;	
 
	INSERT INTO VOTO_BLANCO(DESCRIPCION)
	VALUES('Voto en Blanco');
        
	RSP := 'OK, se ha dado de alta el voto en blanco correctamente';

	p_insertaLog('p_altavotoblanco',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El voto en blanco ya existe';
		ROLLBACK;
		p_insertaLog('p_altavotoblanco',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el voto en blanco';
		ROLLBACK;
		p_insertaLog('p_altavotoblanco',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el voto en blanco: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altavotoblanco',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altapartido
--  OBJETO: Dar de alta los distintos partidos que se presentan en las elecciones
--  PARAMETROS DE ENTRADA:  siglas,nombre,sede
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altapartido (
	siglas in PART_POLITICO.SIGLAS%TYPE,
	nombre in PART_POLITICO.NOMBRE%TYPE,
        sede in PART_POLITICO.SEDE%TYPE,
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  
  obligatorio EXCEPTION;
  
  

  codsiglas PART_POLITICO.SIGLAS%TYPE;  
  codnombre PART_POLITICO.NOMBRE%TYPE;

  BEGIN
	parametros:=NULL;
	


	codsiglas:=upper(siglas);
	codnombre:=upper(nombre);
	

	parametros := siglas||','||nombre||','||sede;


	IF (codsiglas IS NULL OR codsiglas = '') or
	    (codnombre is NULL or codnombre='')
	 THEN
		 RAISE obligatorio;
	END IF;
 
	INSERT INTO PART_POLITICO(siglas,nombre,sede)
	VALUES(codsiglas,codnombre,sede);
        
	RSP := 'OK, se ha dado de alta el partido politico correctamente';

	p_insertaLog('p_altapartido',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El Partido Politico ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altapartido',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el Partido Politico';
		ROLLBACK;
		p_insertaLog('p_altapartido',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altapartido',SYSDATE, parametros, RSP);
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Partido politico: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altapartido',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editapartido
--  OBJETO: Modificar los distintos partidos que se presentan en las elecciones siempre que no hayan participado en otra votacion que ya ha sido publicada
--  PARAMETROS DE ENTRADA:  partido,v_siglas,V_nombre,v_sede
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------


CREATE OR REPLACE
PROCEDURE p_editapartido (
	partido in PART_POLITICO.COD_PARTIDO%TYPE,
	v_siglas in PART_POLITICO.SIGLAS%TYPE,
	v_nombre in PART_POLITICO.NOMBRE%TYPE,
	v_sede in PART_POLITICO.SEDE%TYPE,
        RSP OUT Varchar2
) IS

  
  noencontrado EXCEPTION;
  obligatorio EXCEPTION;
  votacion EXCEPTION;

  existe number:=null;

  
  parametros varchar2(250);

  BEGIN	
  
        
	parametros := partido ||','||v_siglas ||','||v_nombre ||','|| v_sede;

-- Compruebo que el codigo de partido es distinto de vacio o nulo

	IF (partido IS NULL OR partido = '') THEN
		 RAISE obligatorio;
	END IF;

-- Compruebo que el partido existe
        SELECT COUNT(cod_partido) into existe FROM PART_POLITICO WHERE cod_partido=partido;

	IF (existe<1) THEN 
		RAISE noencontrado;
	END IF;

-- Compruebo si existe alguna votacin en la que haya participado dicho partido y la votacion ya haya sido publicada
	SELECT count(candidatos_votacion_partido.cod_votacion) into existe FROM CANDIDATOS_VOTACION_PARTIDO, VOTACION WHERE votacion.cod_votacion=candidatos_votacion_partido.cod_votacion and (estado!=1)
                                                                    and candidatos_votacion_partido.cod_partido=partido;
	
	IF (existe>0) THEN 
		RAISE votacion;
	END IF;


	-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro
	UPDATE PART_POLITICO SET siglas = DECODE(v_siglas,'',siglas,NULL,siglas,v_siglas),nombre=DECODE(v_nombre,'',nombre,NULL,nombre,v_nombre),sede=DECODE(v_sede,'',sede,NULL,sede,v_sede)
					   WHERE cod_partido = partido;	 	


	RSP := 'OK, se ha modificado el Partido  correctamente';

	p_insertaLog('p_editapartido',SYSDATE,parametros, RSP);
	
	COMMIT;


  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El partido ya ha sido dado de alta con otro codigo';
		ROLLBACK;
		p_insertaLog('p_altaestado',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el partido';
		ROLLBACK;
		p_insertaLog('p_editapartido',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editapartido',SYSDATE, parametros, RSP);
		commit;

                
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El Partido no existe';
		ROLLBACK;
		p_insertaLog('p_editapartido',SYSDATE, parametros, RSP);	

	WHEN votacion THEN
	 	RSP := 'ERROR: El Partido no se puede modificar ya que ha participado en alguna votacion que ya ha sido publicada';
		ROLLBACK;
		p_insertaLog('p_editapartido',SYSDATE, parametros, RSP);	
	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar el partido: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editapartido',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajapartido
--  OBJETO: Dar de baja un partido politico
--  PARAMETROS DE ENTRADA:v_partido
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- Si existe un partido asignado a una votacion que ya ha sido publicada no lo podemos eliminar

CREATE OR REPLACE
PROCEDURE p_bajapartido (
 	v_partido in PART_POLITICO.COD_PARTIDO%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;


aux number(1);
total integer;

parametros VARCHAR2(500);


BEGIN

 	parametros := v_partido;
	
	IF (v_partido IS NULL OR v_partido = 0) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que el partido existe
       select count(cod_partido) into aux from part_politico where cod_partido=v_partido;
       if (aux<1) then
          raise noencontrado;
       end if;


-- Comprobamos que el partido no esta asignado a ninguna votacion 

	select count(cod_votacion) into total from opciones_votacion where cod_opcion=v_partido;
	if (total>0) then
		raise nomodificar;
	end if; 


	DELETE OPCIONES WHERE COD_OPCION = v_partido;


	RSP := 'OK, se ha dado de baja el partido correctamente';
	p_insertaLog('p_bajapartido',SYSDATE,parametros, RSP);
	COMMIT;
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: el partido no existe';
		ROLLBACK;
		p_insertaLog('p_bajapartido',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajapartido',SYSDATE, parametros, RSP);	
		commit;


	WHEN nomodificar THEN
	
		RSP := 'ERROR: El partido esta asignado a una votacion, no se puede eliminar';
		ROLLBACK;
		p_insertaLog('p_bajapartido',SYSDATE, parametros, RSP);	
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja el partido: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajacandvotpart',SYSDATE, parametros, RSP);	
		commit;

END;

/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altaelector
--  OBJETO: Da de alta un elector a la vez que le inscribe en el censo regional y local, y actualiza los electores de las votaciones
--  PARAMETROS DE ENTRADA:  elector, region, localidad, fechanac, centro, mesa
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_altaelector (
        elector in ELECTOR.COD_ELECTOR%TYPE,
	region in ELECTOR.COD_CREGIONAL%TYPE,
	localidad in ELECTOR.COD_CLOCAL%TYPE,
        fechanac in ELECTOR.FECHA_NAC%TYPE,
	centro in ELECTOR.COD_CENTRO%TYPE,
	mesa in ELECTOR.COD_MESA%TYPE,
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  paisreg varchar2(10);
  paisloc varchar2(10);
  reg varchar2(10);
  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  noextranjero EXCEPTION;

  existe NUMBER:=NULL;
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);
  PRAGMA EXCEPTION_INIT(valorincorrecto, -02291);
  menoredad EXCEPTION;
  

  BEGIN


	parametros := elector||','||region||','||localidad||','||fechanac||','||centro||','||mesa;


	-- COMPROBAMOS QUE EL ELECTOR TIENE LA EDAD PARA VOTAR (18AOS) QUE EQUIVALE A 216 MESES

	IF (MONTHS_BETWEEN(SYSDATE,FECHANAC)<216) THEN
		RAISE MENOREDAD;
	END IF;	
        
     -- comprobamos que la region existe  
      select count(cod_cregional) into existe from censo_regional where cod_cregional=region;
      if (existe<1) then
          raise valorincorrecto;
      end if;
      
     -- comprobamos que la localidad existe  
      select count(cod_clocal) into existe from censo_local where cod_clocal=localidad;
      if (existe<1) then
          raise valorincorrecto;
      end if;


-- No tenemos que comprobar que la localidad pertenece a la region. Puede ser un aleman residiendo en Espaa, votar a las elecciones municipales de la ciudad espaola, pero
-- votar a las elecciones europeas, autonomicas y regionales donde este inscrito en su pas de origen-

-- Si la region donde se inscribe pertenece al mismo pais que la localidad donde se inscribe esto significa que no es extranjero y por lo tanto si que tenemos que 
-- comprobar que la localidad pertenece a esa region.
   
-- 1. Comprobar si el pais de la region y la localidad es el mismo.

        SELECT cod_pais INTO paisreg FROM CENSO_REGIONAL WHERE cod_cregional=region;
        SELECT cod_pais INTO paisloc FROM CENSO_LOCAL WHERE cod_clocal=localidad;
        
-- 2. Comprobar que la localidad pertenece a esa region.
        IF (paisreg=paisloc) THEN
          -- obtengo la region de ese censo regional
          SELECT cod_region INTO reg FROM censo_regional WHERE cod_cregional=region;
          select count(cod_clocal) into existe from censo_local where ((cod_region=reg) and (cod_clocal=localidad));
          if (existe<1)then
            raise noextranjero;
          end if;
        END IF;

	INSERT INTO ELECTOR(cod_elector,cod_cregional,cod_clocal,fecha_nac,cod_centro,cod_mesa)
	VALUES(elector,region,localidad,fechanac,centro,mesa);
        
	RSP := 'OK, se ha dado de alta el elector correctamente, y se le ha asignado el centro y mesa donde puede ir a votar y los censos a los que pertenece.';

	p_insertaLog('p_altaelector',SYSDATE,parametros, RSP);
	

        
        -- aadimos un elector a todas aquellas votaciones que no esten ni abiertas ni finalizadas que tengan habilitadas el censo en el que se inscribe dicho elector
       p_inelector(region,localidad,0,RSP);
       
       COMMIT;
        
  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El Elector ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el Elector';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;

	WHEN menoredad THEN
	
		RSP := 'ERROR: El ciudadano es menor de edad, no puede ser elector';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;
                
        WHEN noextranjero THEN
	
		RSP := 'ERROR: Si el ciudadano no es extranjero, la localidad debe pertenecer a la region donde esta inscrito';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;
                
	WHEN valorincorrecto THEN
		
		-- Compruebo que la region ha sido dado de alta previamente
	
		SELECT COUNT(cod_cregional) into existe FROM CENSO_REGIONAL WHERE cod_cregional=region;

		IF (existe<1) THEN 
			RSP := 'ERROR: La region no existe, o no ha sido dada de alta previamente';
		END IF;
	
			
 		-- Compruebo que la localidad ha sido dado de alta previamente
		
		SELECT COUNT(cod_clocal) into existe FROM CENSO_LOCAL WHERE cod_clocal=localidad;

		IF (existe<1) THEN 
			RSP := 'ERROR: La localidad no existe, o no ha sido dada de alta previamente';
		END IF;

		-- Compruebo que el Centro y la mesa han sido dados de alta previamente
		
		SELECT COUNT(cod_mesa) into existe FROM MESA WHERE cod_centro=centro and cod_mesa=mesa;

		IF (existe<1) THEN 
			RSP := 'ERROR: El centro o la mesa no existen,no han sido dados de alta previamente';
		END IF;

		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Elector: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;
	
END;



/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editaelector
--  OBJETO: modifica los datos y los censos electorales que tiene asignados un ciudadano
--  PARAMETROS DE ENTRADA: elector,cregional,clocal,nacimiento,centro,mesa 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------


create or replace PROCEDURE p_editaelector (
	v_elector in ELECTOR.COD_ELECTOR%TYPE,
	cregional in ELECTOR.COD_CREGIONAL%TYPE,
        clocal in ELECTOR.COD_CLOCAL%TYPE,
	nacimiento in ELECTOR.FECHA_NAC%TYPE,
	centro in ELECTOR.COD_CENTRO%TYPE,
	mesa in ELECTOR.COD_MESA%TYPE,
        RSP OUT Varchar2
) IS

  codregion  ELECTOR.COD_CREGIONAL%TYPE;
  codlocalidad ELECTOR.COD_CLOCAL%TYPE;
  paisreg varchar2(10);
  paisloc varchar2(10);
  reg varchar2(10);
  


  claveajena EXCEPTION;
  noencontrado EXCEPTION;
  obligatorio EXCEPTION;
  menoredad EXCEPTION;
  noextranjero EXCEPTION;
  

  existe number:=null;

  PRAGMA EXCEPTION_INIT(claveajena,-02291);
 

  parametros varchar2(250);

  BEGIN	
  
        
	parametros := v_elector ||','||cregional ||','||clocal ||','||nacimiento||','||centro||','||mesa;

-- Compruebo que el codigo de elector es distinto de cero o nulo

	IF (v_elector IS NULL OR v_elector = '') THEN
		 RAISE obligatorio;
	END IF;

-- Compruebo que el codigo de elector existe
        SELECT COUNT(cod_elector) into existe FROM ELECTOR WHERE cod_elector=v_elector;

	IF (existe<1) THEN 
		RAISE noencontrado;
	END IF;

IF (MONTHS_BETWEEN(SYSDATE,nacimiento)<216) THEN
		RAISE MENOREDAD;
	END IF;	
        


-- Si la region donde se inscribe pertenece al mismo pais que la localidad donde se inscribe esto significa que no es extranjero y por lo tanto si que tenemos que 
-- comprobar que la localidad pertenece a esa region.
   
-- 1. Comprobar si el pais de la region y la localidad es el mismo.

        SELECT cod_pais INTO paisreg FROM CENSO_REGIONAL WHERE cod_cregional=cregional;
        SELECT cod_pais INTO paisloc FROM CENSO_LOCAL WHERE cod_clocal=clocal;
        
-- 2. Comprobar que la localidad pertenece a esa region.
        IF (paisreg=paisloc) THEN
          -- obtengo la region de ese censo regional
          SELECT cod_region INTO reg FROM censo_regional WHERE cod_cregional=cregional;
          select count(cod_clocal) into existe from censo_local where ((cod_region=reg) and (cod_clocal=clocal));
          if (existe<1)then
            raise noextranjero;
          end if;
        END IF;



-- Almaceno los censos regionales y locales anteriores a la modificacion
	select cod_cregional,cod_clocal into codregion,codlocalidad from elector where cod_elector=v_elector;


	-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro
        UPDATE ELECTOR SET cod_cregional=DECODE(cregional,0 ,cod_cregional,NULL,cod_cregional,cregional), cod_clocal=DECODE(clocal,0,cod_clocal,NULL,cod_clocal,clocal), fecha_nac=DECODE(nacimiento,'',fecha_nac,NULL,fecha_nac,nacimiento), cod_centro=DECODE(centro,0,cod_centro,NULL,cod_centro,centro), cod_mesa=DECODE(mesa,'',cod_mesa,NULL,cod_mesa,mesa) where cod_elector=v_elector;


--   Elimino un elector de los censos regionales y locales anteriores a la modificacion
	p_inelector(codregion,codlocalidad,1,RSP);

--   Aado un elector a los censos regionales y locales nuevos
	p_inelector(cregional,clocal,0,RSP);

	RSP := 'OK, se ha modificado el elector correctamente';

	p_insertaLog('p_editaelector',SYSDATE,parametros, RSP);
	
	COMMIT;


  EXCEPTION
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el elector';
		ROLLBACK;
		p_insertaLog('p_editaelector',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editaelector',SYSDATE, parametros, RSP);
		commit;

	WHEN claveajena THEN
	
		RSP := 'ERROR: La mesa electoral de ese centro no existe';
		ROLLBACK;
		p_insertaLog('p_editaelector',SYSDATE, parametros, RSP);
		commit;	
                
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El elector no existe';
		ROLLBACK;
		p_insertaLog('p_editaelector',SYSDATE, parametros, RSP);	

	WHEN menoredad THEN
	
		RSP := 'ERROR: El ciudadano es menor de edad, no puede ser elector';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;

 	WHEN noextranjero THEN
	
		RSP := 'ERROR: Si el ciudadano no es extranjero, la localidad debe pertenecer a la region donde esta inscrito';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;
                
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar el elector: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editaelector',SYSDATE, parametros, RSP);
		commit;
	
END;
/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajaelector
--  OBJETO: Dar de baja un elector
--  PARAMETROS DE ENTRADA:v_elector
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- Al dar de baja un elector lo tenemos que eliminar de los electores de las votaciones que esten en estado 1 o 2. Edicion o pendiente

CREATE OR REPLACE
PROCEDURE p_bajaelector (
 	v_elector in ELECTOR.COD_ELECTOR%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;


aux number(1);
total integer;
v_region ELECTOR.COD_CREGIONAL%TYPE;
v_local ELECTOR.COD_CLOCAL%TYPE;

parametros VARCHAR2(500);


BEGIN

 	parametros := v_elector;
	
	IF (v_elector IS NULL OR v_elector = '') THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que el elector existe
       select count(cod_elector) into aux from elector where cod_elector=v_elector;
       if (aux<1) then
          raise noencontrado;
       end if;

-- obtenemos el censo regional y local para poder eliminar el elector en las elecciones correspondientes
	select cod_cregional,cod_clocal into v_region,v_local from elector where cod_elector=v_elector;

	DELETE ELECTOR WHERE COD_ELECTOR = v_elector;

	commit;

	p_inelector(v_region,v_local,1,RSP);
	
	RSP := 'OK, se ha dado de baja el elector correctamente';
	p_insertaLog('p_bajaelector',SYSDATE,parametros, RSP);
	COMMIT;

	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: el elector no existe';
		ROLLBACK;
		p_insertaLog('p_bajaelector',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajaelector',SYSDATE, parametros, RSP);	
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja el elector: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajaelector',SYSDATE, parametros, RSP);	
		commit;

END;

/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_codigo
--  OBJETO: generar un codigo aleatorio para poder asignarlo a un elector
--  PARAMETROS DE ENTRADA:  
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_codigo (
	
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  codigo varchar2(15);
  

  BEGIN


	parametros := RSP;
	
	select DBMS_RANDOM.STRING('A',15) INTO codigo from dual; 

	DBMS_OUTPUT.PUT_LINE('El codigo aleatorio generado para asignar al ciudadano es: ' || codigo);
        
	RSP := 'OK, se ha generado un codigo aleatorio para poder asignarlo a un elector';

	p_insertaLog('p_codigo',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El Elector ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);	
		commit;
	

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Elector: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altaelector',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacandidato
--  OBJETO: Dar de alta los los candidatos que se presentan en los distintos partidos politicos
--  PARAMETROS DE ENTRADA: candidato,orden,nombre,ape1,ape2
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_altacandidato (
        candidato in CANDIDATO.COD_CANDIDATO%TYPE,
	nombre in CANDIDATO.NOMBRE%TYPE,
        ape1 in CANDIDATO.APELLIDO1%TYPE,
	ape2 in CANDIDATO.APELLIDO2%TYPE,
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;

  existe NUMBER:=NULL;
  aux number(1);
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);
  

  BEGIN


	parametros := candidato||','||nombre||','||ape1||','||ape2;
 
 -- COMPROBAMOS QUE EL CANDIDADTO ES ELECTOR, REQUISITO PARA PODER PRESENTARSE EN UNA CANDIDATURA
        
        select count(cod_elector) into aux from elector where cod_elector=candidato;
        if (aux<1) then
            raise valorincorrecto;
        end if;

	INSERT INTO CANDIDATO(cod_candidato,nombre,apellido1,apellido2)
	VALUES(candidato,nombre,ape1,ape2);
        
	RSP := 'OK, se ha dado de alta el candidato correctamente.';

	p_insertaLog('p_altacandidato',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El candidato ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altacandidato',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el Candidato';
		ROLLBACK;
		p_insertaLog('p_altacandidato',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altacandidato',SYSDATE, parametros, RSP);
		commit;

	WHEN valorincorrecto THEN

	-- COMPROBAMOS QUE EL CANDIDADTO ES ELECTOR, REQUISITO PARA PODER PRESENTARSE EN UNA CANDIDATURA
		
		RSP := 'ERROR: Para poder ser candidato primero tiene que darse de alta como elector, o el codigo es erroneo';
		ROLLBACK;
		p_insertaLog('p_altacandidato',SYSDATE, parametros, RSP);
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta el Candidato ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacandidato',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editacandidato
--  OBJETO: modificar los candidatos que se pueden presentar a los partidos siempre que no hayan participado en una votacion que ya haya sido publicada
--  PARAMETROS DE ENTRADA: v_candidato, v_nombre, orden, ape1,ape2
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------


CREATE OR REPLACE
PROCEDURE p_editacandidato (
	v_candidato in CANDIDATO.COD_CANDIDATO%TYPE,
	v_nombre in CANDIDATO.NOMBRE%TYPE,
	ape1 in CANDIDATO.APELLIDO1%TYPE,
	ape2 in CANDIDATO.APELLIDO2%TYPE,
        RSP OUT Varchar2
) IS

  
  noencontrado EXCEPTION;
  obligatorio EXCEPTION;
  votacion EXCEPTION;

  existe number:=null;

  
  parametros varchar2(250);

  BEGIN	
  
        
	parametros := v_candidato ||','||v_nombre ||','||ape1 ||','|| ape2;

-- Compruebo que el codigo de candidato es distinto de vacio o nulo

	IF (v_candidato IS NULL OR v_candidato = '') THEN
		 RAISE obligatorio;
	END IF;

-- Compruebo que el candidato existe
        SELECT COUNT(cod_candidato) into existe FROM CANDIDATO WHERE cod_candidato=v_candidato;

	IF (existe<1) THEN 
		RAISE noencontrado;
	END IF;

-- Compruebo si existe alguna votacin en la que haya participado dicho candidato y la votacion ya haya sido publicada
	SELECT count(candidatos_votacion_partido.cod_votacion) into existe FROM CANDIDATOS_VOTACION_PARTIDO, VOTACION WHERE votacion.cod_votacion=candidatos_votacion_partido.cod_votacion and (estado!=1)
                                                                    and candidatos_votacion_partido.cod_candidato=v_candidato;
	
	IF (existe>0) THEN 
		RAISE votacion;
	END IF;


	-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro
	UPDATE CANDIDATO SET nombre = DECODE(v_nombre,'',nombre,NULL,nombre,v_nombre),apellido1=DECODE(ape1,'',apellido1,NULL,apellido1,ape1),apellido2=DECODE(ape2,'',apellido2,NULL,apellido2,ape2)
					   WHERE cod_candidato = v_candidato;	 	


	RSP := 'OK, se ha modificado el Candidato correctamente';

	p_insertaLog('p_editacandidato',SYSDATE,parametros, RSP);
	
	COMMIT;


  EXCEPTION

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el Candidato';
		ROLLBACK;
		p_insertaLog('p_editacandidato',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editacandidato',SYSDATE, parametros, RSP);
		commit;

                
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El Candidato no existe';
		ROLLBACK;
		p_insertaLog('p_editacandidato',SYSDATE, parametros, RSP);	

	WHEN votacion THEN
	 	RSP := 'ERROR: El candidato no se puede modificar ya que ha participado en alguna votacion que ya ha sido publicada';
		ROLLBACK;
		p_insertaLog('p_editacandidato',SYSDATE, parametros, RSP);	
	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar el Candidato: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editacandidato',SYSDATE, parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajacandidato
--  OBJETO: Dar de baja un candidato
--  PARAMETROS DE ENTRADA:v_candidato
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- no se puede eliminar un candidato que este asignado en alguna votacion

CREATE OR REPLACE
PROCEDURE p_bajacandidato (
 	v_candidato in CANDIDATO.COD_CANDIDATO%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;


aux number(1);
total integer;

parametros VARCHAR2(500);


BEGIN

 	parametros := v_candidato;
	
	IF (v_candidato IS NULL OR v_candidato = '') THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que el candidato existe
       select count(cod_candidato) into aux from candidato where cod_candidato=v_candidato;
       if (aux<1) then
          raise noencontrado;
       end if;


-- Comprobamos que el candidato no esta asignado a ninguna votacion 

	select count(cod_votacion) into total from candidatos_votacion_partido where cod_candidato=v_candidato;
	if (total>0) then
		raise nomodificar;
	end if; 


	DELETE CANDIDATO WHERE COD_CANDIDATO = v_candidato;


	RSP := 'OK, se ha dado de baja el candidato correctamente';
	p_insertaLog('p_bajacandidato',SYSDATE,parametros, RSP);
	COMMIT;
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: el candidato no existe';
		ROLLBACK;
		p_insertaLog('p_bajacandidato',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajacandidato',SYSDATE, parametros, RSP);	
		commit;


	WHEN nomodificar THEN
	
		RSP := 'ERROR: El candidato esta asignado a una votacion, no se puede eliminar';
		ROLLBACK;
		p_insertaLog('p_bajacandidato',SYSDATE, parametros, RSP);	
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja el candidato: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajacandidato',SYSDATE, parametros, RSP);	
		commit;

END;

/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altavotacion
--  OBJETO: Dar de alta las distintas votaciones que se pueden realizar a nivel de toda la comunidad Europea
--  PARAMETROS DE ENTRADA: titulo,descripcion,fechapubli,fechaini,fechafin,direccion,ambito,presidente
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_altavotacion (
        titulo in VOTACION.TITULO%TYPE,
	descripcion in VOTACION.DESCRIPCION%TYPE,
        fechapubli in VOTACION.FECHA_PUBLICACION%TYPE,
	fechaini in VOTACION.FECHA_INICIO%TYPE,
	fechafin in VOTACION.FECHA_FIN%TYPE,
	direccion in VOTACION.DIRECCION_WEB%TYPE,
	ambito in VOTACION.AMBITO%TYPE,
	presidente in VOTACION.PRESIDENTE%TYPE,
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  electores integer;
  votos_totales integer;
  porcent_participacion number(3,2);
  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  fpublicacion EXCEPTION;
  ckfechas  EXCEPTION;

 
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);
  PRAGMA EXCEPTION_INIT(ckfechas,-02290);
  PRAGMA EXCEPTION_INIT(valorincorrecto,-02291);
  

  BEGIN

	electores:=0;
	votos_totales:=0;
	porcent_participacion:=0;
	
	parametros := titulo||','||descripcion||','||fechapubli||','||fechaini||','||fechafin||','||direccion||','||ambito||','||presidente;
   
	-- Damos de alta la votacion con estado 1 que es en edicion, comprobamos antes de darle de alta que la fecha del sistema sea menor
 	--  o igual a la fecha de publicacion.

	IF ((fechapubli-SYSDATE)<0) THEN
		RAISE fpublicacion;
	END IF; 

	-- Si el ambito es Global actualizo el nmero de electores de esa votacion como el total de todos los ciudadanos inscritos en los censos locales.
        
	IF (ambito=1) then
		select count(cod_elector) into electores from elector;
	END IF;
	
	INSERT INTO VOTACION(titulo,descripcion,fecha_publicacion,fecha_inicio,fecha_fin,estado,direccion_web,electores,votos_totales,porcent_participacion,ambito,presidente)
	VALUES(titulo,descripcion,fechapubli,fechaini,fechafin,1,direccion,electores,votos_totales,porcent_participacion,ambito,presidente);
	
        
	RSP := 'OK, se ha dado de alta la votacion correctamente.';

	p_insertaLog('p_altavotacion',SYSDATE,parametros, RSP);
	
	
	COMMIT;
	
		

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: La votacion ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altavotacion',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar la votacion';
		ROLLBACK;
		p_insertaLog('p_altavotacion',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altavotacion',SYSDATE, parametros, RSP);
		commit;
	
	WHEN fpublicacion THEN
	
		RSP := 'ERROR: La fecha de publicacion tiene que ser posterior a la fecha actual';
		ROLLBACK;
		p_insertaLog('p_altavotacion',SYSDATE, parametros, RSP);
		commit;

	WHEN valorincorrecto THEN
		
		RSP := 'ERROR: El ambito no existe o no esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altavotacion',SYSDATE, parametros, RSP);
		commit;
		

 	WHEN ckfechas THEN
		if (fechaini>fechafin) then
			RSP := 'ERROR: la fecha de inicio de la votacion es mayor que la fecha fin';
			ROLLBACK;	
		end if;
		
		if ((fechaini-fechapubli)<=7) then
			RSP := 'ERROR: la fecha de publicacion tiene que ser minimo una semana anterior a la de inicio';
			ROLLBACK;	
		end if;

		p_insertaLog('p_altavotacion',SYSDATE, parametros, RSP);
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta la votacion ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacandidato',SYSDATE, parametros, RSP);
		commit;
	
END;
/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editavotacion
--  OBJETO: modificar una votacion
--  PARAMETROS DE ENTRADA:v_votacion, v_titulo ,v_descripcion,f_ini,f_publi,f_fin, web, v_ambito, v_presidente
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- La votacin slo se puede modificar si est en estado de edicion
-- Si la modificamos como la fecha de publicacion si se modifica tiene que ser posterior a la fecha actual siempre 
-- estar en estado de edicin.
-- El estado slo se podr modificar automticamente, no lo puede modificar el usuario.


create or replace PROCEDURE p_editavotacion (
	v_votacion in VOTACION.COD_VOTACION%TYPE,
	v_titulo in VOTACION.TITULO%TYPE,
	v_descripcion in VOTACION.DESCRIPCION%TYPE,
	f_publi in VOTACION.FECHA_INICIO%TYPE,
	f_ini in VOTACION.FECHA_PUBLICACION%TYPE,
	f_fin in VOTACION.FECHA_FIN%TYPE,
	web in VOTACION.DIRECCION_WEB%TYPE,
	v_ambito in VOTACION.AMBITO%TYPE,
	v_presidente in VOTACION.PRESIDENTE%TYPE,
        RSP OUT Varchar2
) IS

  
  noencontrado EXCEPTION;
  obligatorio EXCEPTION;
  nomodificar EXCEPTION;
  fpublicacion EXCEPTION;  
  ckfechas  EXCEPTION;
  valorincorrecto EXCEPTION;

  existe number(1);
  v_estado number(1);
  v_electores integer;
  oldambito number(1);


  PRAGMA EXCEPTION_INIT(obligatorio,-01400);
  PRAGMA EXCEPTION_INIT(ckfechas,-02290);
  PRAGMA EXCEPTION_INIT(valorincorrecto,-02291);
  
  
  parametros varchar2(250);

  BEGIN	
  
        v_electores:=0;
        
	parametros := v_votacion ||','||v_titulo ||','||v_descripcion ||','|| f_ini||','||f_publi ||','||f_fin||','||web ||','||v_ambito||','||v_presidente;


-- Compruebo que el codigo de votacion es distinto de vacio o nulo

	IF (v_votacion is NULL or v_votacion='') THEN
		 RAISE obligatorio;
	END IF;

-- Compruebo que la votacion existe
	select count(cod_votacion) into existe from votacion where cod_votacion=v_votacion;
	if (existe<1) then
		raise noencontrado;
	end if;


-- Compruebo si la votacion esta en estado distinto de votacin, en ese caso ya no se puede modificar

	select estado into v_estado from votacion where cod_votacion=v_votacion;
	if (v_estado !=1 ) then
		Raise nomodificar;
	end if;

-- Comprobamos antes de modificar que la fecha del sistema sea superior o igual a la fecha de publicacion.

	IF ((f_publi-SYSDATE)<0) THEN
		RAISE fpublicacion;
	END IF; 


-- Si el ambito es Global actualizo del nmero de electores de esa votacion como el total de todos los ciudadanos inscritos en los censos locales.
	IF (v_ambito=1) then
		select count(cod_elector) into v_electores from elector;
	END IF;

-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro

	UPDATE VOTACION SET titulo =DECODE(v_titulo,'',titulo,NULL,titulo,v_titulo),
			    descripcion=DECODE(v_descripcion,'',descripcion,NULL,descripcion,v_descripcion),
			    fecha_inicio=DECODE(f_ini,'',fecha_inicio,NULL,fecha_inicio,f_ini),
			    fecha_publicacion=DECODE(f_publi,'',fecha_publicacion,NULL,fecha_publicacion,f_publi),
			    fecha_fin=DECODE(f_fin,'',fecha_fin,NULL,fecha_fin,f_fin),
			    direccion_web=DECODE(web,'',direccion_web,NULL,direccion_web,web),
			    ambito=DECODE(v_ambito,0,ambito,NULL,ambito,v_ambito),
			    presidente=DECODE(v_presidente,'',presidente,NULL,presidente,v_presidente),
                            electores=v_electores
                            WHERE cod_votacion = v_votacion;	 	


	RSP := 'OK, se ha modificado la votacion  correctamente';

	p_insertaLog('p_editavotacion',SYSDATE,parametros, RSP);
	
	COMMIT;


  EXCEPTION
  
       WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: La votacion ya esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_altavotacion',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar la votacion correctamente';
		ROLLBACK;
		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);
		commit;
     
	WHEN noencontrado THEN
	 	RSP := 'ERROR: La votacion no existe';
		ROLLBACK;
		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);	


	WHEN nomodificar THEN
	 	RSP := 'ERROR: No se puede modificar porque la votacion ya no esta en edicion';
		ROLLBACK;
		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);	

	WHEN fpublicacion THEN
	
		RSP := 'ERROR: La fecha de publicacion tiene que ser posterior a la fecha actual';
		ROLLBACK;
		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);
		commit;

	WHEN ckfechas THEN
		if (f_ini>f_fin) then
			RSP := 'ERROR: la fecha de inicio de la votacion es mayor que la fecha fin';
			ROLLBACK;	
		end if;
		
		if ((f_ini-f_publi)<=7) then
			RSP := 'ERROR: la fecha de publicacion tiene que ser minimo una semana anterior a la de inicio';
			ROLLBACK;	
		end if;

		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);
		commit;

	WHEN valorincorrecto THEN
		
		RSP := 'ERROR: El ambito no existe o no esta dado de alta';
		ROLLBACK;
		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);
		commit;
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar la votacion: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editavotacion',SYSDATE, parametros, RSP);
		commit;
	
END;

/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajavotacion
--  OBJETO: eliminar una votacion
--  PARAMETROS DE ENTRADA:v_votacion
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

-- Si elimino una votacion se tienen que eliminar todos los censos asignados a la votacion , todas las opciones de votacion asignadas a dicha votacion, y todas las asignaciones de de los candidatos a un partido en dicha votacion
-- siempre y cuando la votacion este en edicion, si la votacion ya ha sido publicada no la podemos eliminar.

CREATE OR REPLACE
PROCEDURE p_bajavotacion (
 	v_votacion in VOTACION.COD_VOTACION%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;

v_estado number(1);
vota number(1);

parametros VARCHAR2(500);


BEGIN

 	parametros := v_votacion;
	
	IF (v_votacion IS NULL OR v_votacion = 0) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que la votacion existe
       select count(cod_votacion) into vota from votacion where cod_votacion=v_votacion;
       if (vota<1) then
          raise noencontrado;
       end if;
       
-- Comprobamos que la votacion  no ha sido ya publicada
	select estado into v_estado from votacion where cod_votacion=v_votacion;
	if (v_estado != 1) then
		raise nomodificar;
	end if; 

	DELETE VOTACION WHERE COD_VOTACION = v_votacion;
	
	RSP := 'OK, se ha dado de baja la votacion correctamente';
	p_insertaLog('p_bajavotacion',SYSDATE,parametros, RSP);
	COMMIT;
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: la votacion no existe';
		ROLLBACK;
		p_insertaLog('p_bajavotacion',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajavotacion',SYSDATE, parametros, RSP);	
		commit;


	WHEN nomodificar THEN
	
		RSP := 'ERROR: No se puede dar de baja una votacion que ya ha sido publicada';
		ROLLBACK;
		p_insertaLog('p_bajavotacion',SYSDATE, parametros, RSP);	
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja la votacion: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajavotacion',SYSDATE, parametros, RSP);	
		commit;

END;

/
-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altaopcionesvotacion
--  OBJETO: Asignar a cada votacion las opciones que se pueden votar
--  PARAMETROS DE ENTRADA: votacion,opcion
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_altaopcionesvotacion (
        votacion in OPCIONES_VOTACION.COD_VOTACION%TYPE,
	opcion in OPCIONES_VOTACION.COD_OPCION%TYPE,
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  votopt integer;
  porcentaje number(3,2);
  estado number(1);
  existe number(1);
  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  noedicion EXCEPTION;
  
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);
  PRAGMA EXCEPTION_INIT(valorincorrecto,-02291);
  

  BEGIN
      
        votopt:=0;
        porcentaje:=0;
        
	parametros := votacion||','||opcion;
        
        -- comprobamos que la votacion existe
	select count(cod_votacion) into existe from votacion where cod_votacion=votacion;

	if (existe<1) then
		raise valorincorrecto;
	end if;
        
        -- Comprobamos que el estado de la votacion es edicion, en caso contario erro
        select votacion.estado into estado from votacion where cod_votacion=votacion;
        
        if (estado!= 1) then
          raise noedicion;
        end if;

	INSERT INTO OPCIONES_VOTACION(cod_votacion,cod_opcion,votos_opcion,porcent_participacion)
	VALUES(votacion,opcion,votopt,porcentaje);
        
	RSP := 'OK, se han asignado las opciones de votacion correctamente.';

	p_insertaLog('p_altaopcionesvotacion',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: La opcion ya ha sido asignada a esta votacion';
		ROLLBACK;
		p_insertaLog('p_altaopcionesvotacion',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo asignar la opcion  a la votacion';
		ROLLBACK;
		p_insertaLog('p_altaoopcionesvotacion',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altaopcionesvotacion',SYSDATE, parametros, RSP);
		commit;
	
	
	WHEN valorincorrecto THEN
		
		RSP := 'ERROR: La votacion o la opcion de votacion no existe o es incorrecta';
		ROLLBACK;
		p_insertaLog('p_altaopcionesvotacion',SYSDATE, parametros, RSP);
		commit;
		
      WHEN noedicion THEN
		
		RSP := 'ERROR: La votacion ya no est en edicion, no se pueden aadir ms opciones a la votacion';
		ROLLBACK;
		p_insertaLog('p_altaopcionesvotacion',SYSDATE, parametros, RSP);
		commit;
                
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al asignar las distintas opciones que se pueden votar ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacandidato',SYSDATE, parametros, RSP);
		commit;
	
END;


/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajaopcionvotacion
--  OBJETO: Dejar de asignar una opcion en la votacion
--  PARAMETROS DE ENTRADA:v_votacion,v_opcion
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_bajaopcionvotacion (
 	v_votacion in OPCIONES_VOTACION.COD_VOTACION%TYPE,
	v_opcion in OPCIONES_VOTACION.COD_OPCION%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;


v_estado number(1);
aux number(1);

parametros VARCHAR2(500);


BEGIN

 	parametros := v_votacion|| ',' || v_opcion;
	
	IF ((v_votacion IS NULL OR v_votacion = 0) or (v_opcion is null or v_opcion=0)) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que la votacion existe
       select count(cod_votacion) into aux from votacion where cod_votacion=v_votacion;
       if (aux<1) then
          raise noencontrado;
       end if;

-- Comprobamos que la opcion existe
       select count(cod_opcion) into aux from opciones where cod_opcion=v_opcion;
       if (aux<1) then
          raise noencontrado;
       end if;


-- Comprobamos que la votacion  no ha sido ya publicada
	select estado into v_estado from votacion where cod_votacion=v_votacion;
	if (v_estado != 1) then
		raise nomodificar;
	end if; 


	DELETE OPCIONES_VOTACION WHERE COD_VOTACION = v_votacion AND COD_OPCION=v_opcion;

	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
        END IF;

	
	RSP := 'OK, se ha dado de baja la opcion en la votacion correctamente';
	p_insertaLog('p_bajaopcionvotacion',SYSDATE,parametros, RSP);
	COMMIT;
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: la votacion, o la opcion de votacion no existe';
		ROLLBACK;
		p_insertaLog('p_bajaopcionvotacion',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajaopcionvotacion',SYSDATE, parametros, RSP);	
		commit;


	WHEN nomodificar THEN
	
		RSP := 'ERROR: No se puede dar de baja la asignacion de la opcion a la votatacion, ya  que esta ya ha sido publicada';
		ROLLBACK;
		p_insertaLog('p_bajaopcionvotacion',SYSDATE, parametros, RSP);	
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja la opcion de la votacion: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajaopcionvotacion',SYSDATE, parametros, RSP);	
		commit;

END;

/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacensoshabilitados
--  OBJETO: habilitar los censos de una votacion y aadir los electores de ese censo a la votacion
--  PARAMETROS DE ENTRADA: votacion, censo
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_altacensoshabilitados (
        votacion in CENSOS_HABILITADOS.COD_VOTACION%TYPE,
	censo in CENSOS_HABILITADOS.COD_CENSO%TYPE,
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  ambitovot number(1);
  ambitocenso number(1);
  existe number:=null;
  estadovot number(1);
  
  valorincorrecto EXCEPTION;
  censoshabilitados EXCEPTION;
  censoambitoerroneo EXCEPTION;
  nomodificar EXCEPTION;
  
  
  PRAGMA EXCEPTION_INIT(valorincorrecto,-02291);
  

  BEGIN

	parametros := votacion||','||censo;
	
	

--	comprobamos que el codigo de votacion ha sido dado de alta previamente
	
	SELECT COUNT(cod_votacion) into existe FROM VOTACION WHERE cod_votacion=votacion;

	IF (existe<1) THEN 
		RAISE valorincorrecto;
	END IF;


--	comprobamos que el codigo de censo ha sido dado de alta previamente
	
	SELECT COUNT(cod_censo) into existe FROM CENSO WHERE cod_censo=censo;

	IF (existe<1) THEN 
		RAISE valorincorrecto;
	END IF;
        
--      comprobamos que la votacion no este publicada, una vez publicada ya no se pueden cambiar los censos que tiene asociados dicha votacion
        SELECT estado into estadovot FROM VOTACION WHERE cod_votacion=votacion;

        
         if (estadovot =3 or estadovot=4) then
          raise nomodificar;
        end if;

-- Comprobar que el censo que vamos a habilitar corresponde al ambito de esa votacion
   	SELECT ambito into ambitovot FROM VOTACION WHERE cod_votacion=votacion;
	
	if (ambitovot=1) then

		  RAISE censoshabilitados;

	else
		select tipo_censo into ambitocenso FROM CENSO WHERE cod_censo=censo;
		
		if (ambitocenso!= ambitovot) then

		  RAISE censoambitoerroneo;

		end if;
	end if;

	INSERT INTO CENSOS_HABILITADOS(cod_votacion,cod_censo)
	VALUES(votacion,censo);
        
	RSP := 'OK, se ha habilitado el censo de votacion correctamente.';

	p_insertaLog('p_altacensoshabilitados',SYSDATE,parametros, RSP);


-- Inicializamos el campo electores con el nmero de personas que hay inscritas en el censo de dicha votacion.

	p_insertaelectores(votacion,ambitovot,censo,0,RSP);

	  -- actualizo estadistica de numero de votaciones asociadas a un censo durante un ao
        delete est_censo_anyo_vota;
	insert into est_censo_anyo_vota (select cod_censo, extract(year from fecha_inicio) ao, count(cod_votacion) total   from votacenso group by cod_censo, extract(year from fecha_inicio));
	
	
	COMMIT;



	

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El censo ya ha sido habilitado a esta votacion';
		ROLLBACK;
		p_insertaLog('p_altacensoshabilitados',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo habilitar el censo a la votacion';
		ROLLBACK;
		p_insertaLog('p_altacensoshabilitados',SYSDATE, parametros, RSP);	
		commit;
	
	
	
	WHEN valorincorrecto THEN
		
		RSP := 'ERROR: La votacion o el censo no existe o es incorrecto';
		ROLLBACK;
		p_insertaLog('p_altacensoshabilitados',SYSDATE, parametros, RSP);
		commit;
	
	WHEN censoshabilitados THEN
		
		RSP := 'ERROR: El censo global ya ha sido habilitado para esta votacion';
		ROLLBACK;
		p_insertaLog('p_altacensoshabilitados',SYSDATE, parametros, RSP);
		commit;

	WHEN censoambitoerroneo THEN
		
		RSP := 'ERROR: El censo que se pretende habilitar no es del ambito de esta votacion';
		ROLLBACK;
		p_insertaLog('p_altacensoshabilitados',SYSDATE, parametros, RSP);
		commit;
        
        WHEN nomodificar THEN
		
		RSP := 'ERROR: La votacion una vez abierta no se le pueden aadir censos';
		ROLLBACK;
		p_insertaLog('p_altacensoshabilitados',SYSDATE, parametros, RSP);
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al habilitar los distintas censos a la votacion ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacensoshabilitados',SYSDATE, parametros, RSP);
		commit;
	
END;

/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajacensohabilitado
--  OBJETO: Dejar de asignar a una votacion un censo
--  PARAMETROS DE ENTRADA:v_votacion,v_censo
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_bajacensohabilitado (
 	v_votacion in CENSOS_HABILITADOS.COD_VOTACION%TYPE,
	v_censo in CENSOS_HABILITADOS.COD_CENSO%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;
noambito EXCEPTION;

aux number(1);

parametros VARCHAR2(500);


BEGIN

 	parametros := v_votacion|| ',' || v_censo;
	
	IF ((v_votacion IS NULL OR v_votacion = 0) or (v_censo is null or v_censo=0)) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que la votacion existe
       select count(cod_votacion) into aux from votacion where cod_votacion=v_votacion;
       if (aux<1) then
          raise noencontrado;
       end if;

-- Comprobamos que el censo existe
       select count(cod_censo) into aux from censo where cod_censo=v_censo;
       if (aux<1) then
          raise noencontrado;
       end if;


-- Comprobamos que la votacion  no ha sido ya publicada
	select estado into aux from votacion where cod_votacion=v_votacion;
	if (aux=3 or aux=4) then
		raise nomodificar;
	end if; 

-- Comprobamos que el ambito de la votacion no es global, ya que en este caso deben estar asignados todos los censos nacionales, no se puede eliminar ninguna asignacion
	
	select ambito into aux from votacion where cod_votacion=v_votacion;
	if (aux = 1) then
		raise noambito;
	end if; 


	DELETE CENSOS_HABILITADOS WHERE COD_VOTACION = v_votacion AND COD_CENSO=v_censo;

        IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
        END IF;

	commit;

	p_insertaelectores(v_votacion,aux,v_censo,1,RSP);
	
	
	  -- actualizo estadistica de numero de votaciones asociadas a un censo durante un ao
        delete est_censo_anyo_vota;
	insert into est_censo_anyo_vota (select cod_censo, extract(year from fecha_inicio) ao, count(cod_votacion) total   from votacenso group by cod_censo, extract(year from fecha_inicio));
	

	RSP := 'OK, se ha dado de baja el censo en la votacion correctamente';
	p_insertaLog('p_bajacensohabilitado',SYSDATE,parametros, RSP);
	COMMIT;
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: la votacion, o el censo de  la votacion no existe';
		ROLLBACK;
		p_insertaLog('p_bajacensohabilitado',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajacensohabilitado',SYSDATE, parametros, RSP);	
		commit;


	WHEN nomodificar THEN
	
		RSP := 'ERROR: La votacion una vez abierta no se le pueden dar de baja censos';
		ROLLBACK;
		p_insertaLog('p_bajacensohabilitado',SYSDATE, parametros, RSP);	
		commit;

	WHEN noambito THEN
	
		RSP := 'ERROR: No se puede dejar de asignar un censo cuando la votacion es de ambito global, todos los paises de la UE';
		ROLLBACK;
		p_insertaLog('p_bajacensohabilitado',SYSDATE, parametros, RSP);	
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja el censo habilitado de la votacion: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajacensohabilitado',SYSDATE, parametros, RSP);	
		commit;

END;

/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altacandivotpartido
--  OBJETO: Asignar a cada votacion los candidatos que se presentan por cada partido
--  PARAMETROS DE ENTRADA: votacion,partido,candidato
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

create or replace PROCEDURE p_altacandivotpartido (
        votacion in CANDIDATOS_VOTACION_PARTIDO.COD_VOTACION%TYPE,
	partido in CANDIDATOS_VOTACION_PARTIDO.COD_PARTIDO%TYPE,
	candidato in CANDIDATOS_VOTACION_PARTIDO.COD_CANDIDATO%TYPE,
	lista in CANDIDATOS_VOTACION_PARTIDO.ORDEN_LISTA%TYPE,
	RSP OUT Varchar2
) IS

  parametros varchar2(250);
  estado number(1);
  existe number(1);
  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  noedicion EXCEPTION;
  nopartido EXCEPTION;
  
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);
  PRAGMA EXCEPTION_INIT(valorincorrecto,-02291);
  

  BEGIN

	parametros := votacion||','||partido||','||candidato||','||lista;
        
        if ((votacion is NULL or votacion='') or (partido is NULL or partido='') or (candidato is NULL or candidato=''))then
          raise obligatorio;
        end if;
         -- comprobamos que la votacion existe
	select count(cod_votacion) into existe from votacion where cod_votacion=votacion;

	if (existe<1) then
		raise valorincorrecto;
	end if;
        
        -- Comprobamos que el candidato existe
        select count(cod_candidato) into existe from candidato where cod_candidato=candidato;
        if (existe<1) then
            raise valorincorrecto;
        end if;
        
        -- Comprobamos que el estado de la votacion es edicion, en caso contario erro
        select votacion.estado into estado from votacion where cod_votacion=votacion;
         if (estado != 1) then
          raise noedicion;
        end if;
        
        -- Comprobamos que el partido esta asignado a la votacion
        select count(cod_votacion) into existe from opciones_votacion where cod_opcion=partido and cod_votacion=votacion;
        if(existe<1)then
          raise nopartido;
        end if;
        
	INSERT INTO CANDIDATOS_VOTACION_PARTIDO(cod_votacion,cod_partido,cod_candidato,orden_lista)
	VALUES(votacion,partido,candidato,lista);
        
	RSP := 'OK, se han asignado los candidatos a los partidos que se presenta en la votacion correctamente.';

	p_insertaLog('p_altacandidatosvotacionpartido',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: Ya se ha asignado este candidato a un partido en la misma votacion o ya se ha presentado un candidato para ese numero de puesto en la opcion de votacion';
		ROLLBACK;
		p_insertaLog('p_altacandivotpartido',SYSDATE,parametros, RSP);
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo asignar el candidato al partido politico en la votacion';
		ROLLBACK;
		p_insertaLog('p_altacandivotpartido',SYSDATE,parametros, RSP);
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altacandivotpartido',SYSDATE,parametros, RSP);
		commit;
	
	
	WHEN valorincorrecto THEN
		
		RSP := 'ERROR: La votacion, el candidato o el partido no existe o es incorrecto';
		ROLLBACK;
		p_insertaLog('p_altacandivotpartido',SYSDATE,parametros, RSP);
		commit;
	
        WHEN noedicion THEN
		
		RSP := 'ERROR: La votacion ya no est en edicion, no se pueden asignar candidatos';
		ROLLBACK;
		p_insertaLog('p_altacandivotpartido',SYSDATE,parametros, RSP);
		commit;
			
       WHEN nopartido THEN
		
		RSP := 'ERROR: El partido no se presenta en esta votacion';
		ROLLBACK;
		p_insertaLog('p_altacandivotpartido',SYSDATE,parametros, RSP);
		commit;

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al asignar el candidato al partido en la votacion' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altacandivotpartido',SYSDATE,parametros, RSP);
		commit;
	
END;

/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editacandvotpart
--  OBJETO: modificar el orden de la lista en las que se presentan los candidatos de un partido en una votacion
--  PARAMETROS DE ENTRADA:votacion, partido,candidato,lista
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------


CREATE OR REPLACE
PROCEDURE p_editacandvotpart (
	votacion in CANDIDATOS_VOTACION_PARTIDO.COD_CANDIDATO%TYPE,
	partido in CANDIDATOS_VOTACION_PARTIDO.COD_PARTIDO%TYPE,
	candidato in CANDIDATOS_VOTACION_PARTIDO.COD_CANDIDATO%TYPE,
	lista in CANDIDATOS_VOTACION_PARTIDO.ORDEN_LISTA%TYPE,
        RSP OUT Varchar2
) IS

  

  obligatorio EXCEPTION;
  vota EXCEPTION;
  nomodificar EXCEPTION;
  valorincorrecto EXCEPTION;
  nopartido EXCEPTION;
  
 PRAGMA EXCEPTION_INIT(valorincorrecto,-02291);
 
  existe number(1);
  v_estado number(1);

  
  parametros varchar2(250);

  BEGIN	
  
        
	parametros := votacion ||','||partido ||','||candidato ||','|| lista;

-- Compruebo que el codigo de partido  y candidato es distinto de vacio o nulo

	IF (votacion is NULL or votacion='') or (candidato is NULL or candidato='') THEN
		 RAISE obligatorio;
	END IF;
        
-- Compruebo que la votacion existe
      select count(cod_votacion) into existe from votacion where cod_votacion=votacion;
      if (existe<1) then
        raise valorincorrecto;
      end if;
      
    
-- Comprobamos que el candidato existe
        select count(cod_candidato) into existe from candidato where cod_candidato=candidato;
        if (existe<1) then
            raise valorincorrecto;
        end if;

-- Comprobamos que el partido existe
       select count(cod_partido) into existe from part_politico where cod_partido=partido;
       if(existe<1) then
            raise valorincorrecto;
       end if;
       
-- Compruebo si la votacion esta en estado distinto de edicion, en ese caso ya no se puede modificar
	select estado into v_estado from votacion where cod_votacion=votacion;
	if (v_estado!=1) then
		raise nomodificar;
	end if;

-- Comprobamos que el partido esta asignado a la votacion
        select count(cod_votacion) into existe from opciones_votacion where cod_opcion=partido and cod_votacion=votacion;
        if(existe<1)then
          raise nopartido;
        end if;
     


	-- Si el valor es nulo o vacio introduce el valor anterior de la tabla en caso contrario el valor nuevo que pasamos como parmetro
	UPDATE CANDIDATOS_VOTACION_PARTIDO SET cod_partido = DECODE(partido,0,cod_partido,NULL,cod_partido,partido),orden_lista=DECODE(lista,0,orden_lista,NULL,orden_lista,lista)
					   WHERE cod_votacion = votacion and cod_candidato=candidato;	 	


	RSP := 'OK, se ha modificado la asignacion de los candidatos al partido en la votacion  correctamente';

	p_insertaLog('p_editacandvotpart',SYSDATE,parametros, RSP);
	
	COMMIT;


  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: Existe un candidato en esa posicion de la lista en el partido politico para esa votacion ';
		ROLLBACK;
		p_insertaLog('p_editacandvotpart',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar la asignacion de los candidatos a el partido en la votacion';
		ROLLBACK;
		p_insertaLog('p_editacandvotpart',SYSDATE, parametros, RSP);	
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_editacandvotpart',SYSDATE, parametros, RSP);
		commit;
                
       WHEN valorincorrecto THEN
	
		RSP := 'ERROR: El codigo de votacion, partido o candidato es incorrecto';
		ROLLBACK;
		p_insertaLog('p_editacandvotpart',SYSDATE, parametros, RSP);
		commit;


	WHEN nomodificar THEN
	 	RSP := 'ERROR: No se puede modificar porque la votacion ya no esta en edicion';
		ROLLBACK;
		p_insertaLog('p_editacandvotpart',SYSDATE, parametros, RSP);
                
    		
        WHEN nopartido THEN
		
		RSP := 'ERROR: El partido no se presenta en esta votacion';
		ROLLBACK;
		p_insertaLog('p_edtiacandvotpart',SYSDATE,parametros, RSP);
		commit;
	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar la asignacion de los candidatos a un partido en una votacion: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_editacandvotpart',SYSDATE, parametros, RSP);
		commit;
	
END;

/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_bajacandvotpart
--  OBJETO: Dejar de asignar a una votacion un candidato de un partido
--  PARAMETROS DE ENTRADA:v_votacion,v_partido,v_candidato
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_bajacandvotpart (
 	v_votacion in CANDIDATOS_VOTACION_PARTIDO.COD_VOTACION%TYPE,
	v_partido in CANDIDATOS_VOTACION_PARTIDO.COD_PARTIDO%TYPE,
	v_candidato in CANDIDATOS_VOTACION_PARTIDO.COD_CANDIDATO%TYPE,
	RSP OUT Varchar2
) IS


obligatorio EXCEPTION;
noencontrado EXCEPTION;
nomodificar EXCEPTION;


aux number(1);

parametros VARCHAR2(500);


BEGIN

 	parametros := v_votacion|| ',' || v_partido ||','||v_candidato;
	
	IF ((v_votacion IS NULL OR v_votacion = 0) or (v_partido is null or v_partido=0) or (v_candidato is null or v_candidato='')) THEN
		 RAISE obligatorio;
	END IF;

-- Comprobamos que la votacion existe
       select count(cod_votacion) into aux from votacion where cod_votacion=v_votacion;
       if (aux<1) then
          raise noencontrado;
       end if;

-- Comprobamos que la opcion existe
       select count(cod_opcion) into aux from opciones where cod_opcion=v_partido;
       if (aux<1) then
          raise noencontrado;
       end if;

-- Comprobamos que el candidato existe
       select count(cod_candidato) into aux from candidato where cod_candidato=v_candidato;
       if (aux<1) then
          raise noencontrado;
       end if;


-- Comprobamos que la votacion  no ha sido ya publicada
	select estado into aux from votacion where cod_votacion=v_votacion;
	if (aux != 1) then
		raise nomodificar;
	end if; 


	DELETE CANDIDATOS_VOTACION_PARTIDO WHERE COD_VOTACION = v_votacion AND COD_PARTIDO=v_partido AND COD_CANDIDATO=v_candidato;

        IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
        END IF;


	RSP := 'OK, se ha dado de baja el candidato del partido en la votacion correctamente';
	p_insertaLog('p_bajacandvotpart',SYSDATE,parametros, RSP);
	COMMIT;
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: la votacion, el partido o el candidato de  la votacion no existe';
		ROLLBACK;
		p_insertaLog('p_bajacandvotpart',SYSDATE, parametros, RSP);	
		commit;
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_bajacandvotpart',SYSDATE, parametros, RSP);	
		commit;


	WHEN nomodificar THEN
	
		RSP := 'ERROR: No se puede dar de baja la asignacion del candidato del partido en la votacion, ya  que esta ya ha sido publicada';
		ROLLBACK;
		p_insertaLog('p_bajacandvotpart',SYSDATE, parametros, RSP);	
		commit;


	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja la asignacion del candidato del partido en la votacion: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_bajacandvotpart',SYSDATE, parametros, RSP);	
		commit;

END;

/


-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_altavotar
--  OBJETO: Asignar el voto de un elector a un partido en una votacion concreta
--  PARAMETROS DE ENTRADA: votacion,opcion,votante
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------
CREATE OR REPLACE
PROCEDURE p_altavotar (
        votacion in VOTAR.COD_VOTACION%TYPE,
	opcion in VOTAR.COD_OPCION%TYPE,
	votante in VOTAR.COD_VOTANTE%TYPE,
	RSP OUT Varchar2
) IS


 vot_record censos_habilitados%ROWTYPE;
 
 opt_record opciones_votacion%rowtype;
 
 cen_record censo_regional %rowtype;
 
  parametros varchar2(250);
  existe number(1);
  estado number(1);
  estadovot number(1);
  ambitovot number(1);
  tipopt number(1);
  porcentaje number(5,2);
  opt number(1);
  des varchar2(50);
  
  votos integer;
  votos_opcion integer;
  encontrado boolean:=false;
  v_nacion varchar2(2);
  elect integer;
  votopt integer;
  
  
  c_local integer;
  c_regional integer;
  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  novotar EXCEPTION;
  errorcenso EXCEPTION;
  erroropt EXCEPTION;
  
  
  
 -- PRAGMA EXCEPTION_INIT(obligatorio,-01400);
 
  
  
  

 -- obtener los censos que tiene habilitados una votacion
  cursor censos IS
 	Select * from censos_habilitados where cod_votacion=votacion;
  
  cursor cenreg(v_nacion varchar2) IS
 	Select * from censo_regional where cod_pais=v_nacion;
            
  cursor opciones IS
 	Select * from opciones_votacion where cod_votacion=votacion;
  
  
  BEGIN

	parametros := votacion||','||opcion||','||votante;

   	-- comprobamos que la votacion existe

	select count(cod_votacion) into existe from votacion where cod_votacion=votacion;

	if (existe<1) then
		raise valorincorrecto;
	end if;

      -- comprobamos que la opcion de votacion existe

	select count(cod_opcion) into existe from opciones where cod_opcion=opcion;

	if (existe<1) then
		raise valorincorrecto;
	end if;
        
         -- comprobamos que el candidato existe

	select count(cod_elector) into existe from elector where cod_elector=votante;

	if (existe<1) then
		raise valorincorrecto;
	end if;
        
        
        
	--comprobar que la votacion tiene estado abierta para que se pueda votar
	
	select estado, ambito into estadovot,ambitovot from votacion where cod_votacion=votacion;

	if (estadovot !=3) then
		raise novotar;
	end if;
        
        
        
        -- comprobamos que el censo de la votacion corresponde con el censo del ciudadano ya seal el regional o el local
        
        -- Si la votacion es global pueden votar todos los ciudadanos
        -- Si la votacin es Nacional estaran habilitados todos los censos regionales de los paises que tenga habilitados
                
        -- obtengo el censo local y regional del votante
          select cod_cregional, cod_clocal into  c_regional,c_local from elector where cod_elector=votante;
          
        --Buscar en censos habilitados todos los censos que se encuentran habilitados en esa votacion.
        if (ambitovot=1) then
          encontrado:=true;
        else
        
        open censos; -- Abro el cursor con los censos habilitados para esa votacion
	    --Por cada censo habilitado de la votacion compruebo si coincide con el censo local o regional del votante
                    loop
                        fetch censos into vot_record;
                        exit when censos%notfound;
                            -- ambito Nacional
                            if (ambitovot=2) then
                                    select cod_pais into v_nacion from censo_nacional where cod_cnacional=vot_record.cod_censo;
                                    -- busco los censos regionales de ese pais
                                    open cenreg(v_nacion);
                                      loop
                                          fetch cenreg into cen_record;
                                            exit when cenreg%notfound;
                                            if (cen_record.cod_cregional=c_regional) then
                                              encontrado:=true;
                                              exit;
                                            end if;
                                      end loop;
                                    close cenreg;
                            
                            
                            elsif ((vot_record.cod_censo=c_regional) or (vot_record.cod_censo=c_local)) then
                                encontrado:=true;
                            end if;
		     end loop;
	   close censos; 
      
        end if;
        
        if (encontrado=false) then
          raise errorcenso;
        end if;
        
        encontrado:=false;
       -- Comprobar que la opcion de votacion que ha elegido el votante este habilitada en esa votacion
       
       
       -- buscar las opciones que tiene habilitadas esa votacion
        open opciones; -- Abro el cursor con las opciones habilitadas para esa votacion
	    --Por cada opcion habilitada de la votacion compruebo si coincide con la opcion del votante
                    loop
                        fetch opciones into opt_record;
                        exit when opciones%notfound;
                            if opt_record.cod_opcion=opcion then
                                encontrado:=true;
                                exit;
                            end if;
		     end loop;
	   close opciones; 
        if (encontrado=false) then
          raise erroropt;
        end if;
       
       
        
	INSERT INTO VOTAR(cod_votacion,cod_opcion,cod_votante)
	VALUES(votacion,opcion,votante);
        

        -- obtengo el numero de votos de esa votacion y le aado 1
        select votos_totales into votos from votacion where cod_votacion=votacion;
        votos:=votos +1;
  
  
        -- actualizo el numero de votos de la votacion
        update votacion set votos_totales=votos where cod_votacion=votacion;
        
        
        -- recupero el numero de electores de la votacion
        select electores into elect from votacion where cod_votacion=votacion;
       
        -- calculo el porcentaje de participacion
        select round((votos/elect)*100,2) into porcentaje from dual;
        
        
        -- actualizo el porcentaje de participacion de la votacion
        update votacion set porcent_participacion=porcentaje where cod_votacion=votacion;
    
        -- obtengo el numero de votos de la opcion que ha votado el ciudadano en esa votacion y le aado 1
        select votos_opcion into votopt from opciones_votacion where cod_votacion=votacion and cod_opcion=opcion;
        votopt:=votopt +1;
    
        -- actualizo el numero de votos en esa votacion de la opcion que ha elegido el ciudadano
        update opciones_votacion set votos_opcion=votopt where cod_votacion=votacion and cod_opcion=opcion;
        
        -- Cada vez que se emite un voto hay que actualizar el porcentaje de participacion  de todos los partidos que se presentan por esa votacion
       
         -- buscar las opciones que tiene habilitadas esa votacion
        open opciones; -- Abro el cursor con las opciones habilitadas para esa votacion
	    --Por cada opcion habilitada de la votacion actualizo los porcentajes de participacion
                    loop
                        fetch opciones into opt_record;
                        exit when opciones%notfound;
                                -- actualizo el porcentaje de participacion para esa opcion en concreto
                                select round((opt_record.votos_opcion/votos)*100,2) into porcentaje from dual;
                                update opciones_votacion set porcent_participacion=porcentaje where cod_votacion=opt_record.cod_votacion and cod_opcion=opt_record.cod_opcion;
		     end loop;
	   close opciones; 
      
        -- actualizo en esa votacion la opcion ms votada 
        select cod_opcion,porcent_participacion into opt,porcentaje from (select * from opciones_votacion where cod_votacion=votacion order by porcent_participacion desc) 
        where rownum <2;
        
        select descripcion into des from (select cod_vblanco codigo, descripcion from voto_blanco
        union
        select cod_vnulo, descripcion from voto_nulo
        union
        select cod_partido,nombre from part_politico) where codigo=opt;

        update votacion set optmasvotada=des, porcent_ganadora= porcentaje where cod_votacion=votacion;
        
        -- actualizo en esa votacion la opcion menos votada
        select cod_opcion, porcent_participacion into opt,porcentaje from (select * from opciones_votacion where cod_votacion=votacion order by porcent_participacion) 
        where rownum <2;
        
        select descripcion into des from (select cod_vblanco codigo, descripcion from voto_blanco
        union
        select cod_vnulo, descripcion from voto_nulo
        union
        select cod_partido,nombre from part_politico) where codigo=opt;
        
        update votacion set optmenosvotada=des, porcent_perdedora=porcentaje where cod_votacion=votacion;
	
	commit;

        -- actualizo estadisticas
	p_rellenaestadisticas;

	RSP := 'OK, la votacion se ha realizado con exito.';

	p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El ciudadano ya ha votado en esta votacion';
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo almacenar el voto';
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;
	
	
	WHEN valorincorrecto THEN
		
		RSP := 'ERROR: La votacion,la opcion de votacion o el codigo del candidato no existe o es incorrecto';
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;
	
	WHEN novotar THEN
		
		RSP := 'ERROR: Solo se puede votar si la votacion esta abierta';
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;

        WHEN errorcenso THEN
		
		RSP := 'ERROR: El ciudadano no puede votar, pertenece a otro censo electoral';
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;
                
        WHEN erroropt THEN
		
		RSP := 'ERROR: El ciudadano no puede elegir esa opcion de voto, al no estar dada de alta en la votacion';
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;
                
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al votar' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_altavotar',SYSDATE,parametros, RSP);
		commit;
	
END;


/
-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_insertaelectores
--  OBJETO: Se ejecuta al habilitar un censo en una votacion y aade a los electores de dicho censo a la votacion
--  PARAMETROS DE ENTRADA: votacion, ambito, censo 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_insertaelectores (
        votacion in VOTACION.COD_VOTACION%TYPE,
	ambito in VOTACION.AMBITO%TYPE,
	censo in CENSOS_HABILITADOS.COD_VOTACION%TYPE,
        operacion number,
	RSP OUT Varchar2
) IS
-- si operacion es 0 sumo electores del censo a la votacion
-- si operacion es 1 resto electores del censo a la votacion

  parametros varchar2(250);
  elect integer;
  totalelect integer;
  v_pais varchar2(2);
  
  obligatorio EXCEPTION;
  valorincorrecto EXCEPTION;
  
  
  PRAGMA EXCEPTION_INIT(obligatorio,-01400);
  PRAGMA EXCEPTION_INIT(valorincorrecto,-02291);

  regiones_record CENSO_REGIONAL%ROWTYPE;
 


  -- cursor para obtener las regiones de un pais concreto
  cursor censoregiones (v_pais varchar2) IS
	select * from censo_regional where cod_pais=v_pais;
	
  

  BEGIN
	elect:=0;
	totalelect:=0;

	parametros := votacion||','||ambito||','||censo;

	-- Rellenar el nmero de electores de una votacion segn los censos que tenga habilitados
	-- El censo global ya se ha asignado al dar de alta la votacion 
	
	
	IF (ambito=2) THEN --censos nacionales

		-- obtengo el codigo de pais correspondiente a ese censo nacional
			select cod_pais into v_pais from censo_nacional where cod_cnacional=censo;
			
			open censoregiones(v_pais); -- Abro el cursor con las regiones de un pais
			 --Por cada region cuento el numero de electores inscritos en esa region
                         loop
                            fetch censoregiones into regiones_record;
                                  exit when censoregiones%notfound;
                                  select count(cod_elector) into elect from elector where cod_cregional=regiones_record.cod_cregional;
				  totalelect:=totalelect+elect; 
                                  
			end loop;
			close censoregiones; 
	   
	ELSIF (ambito=3) THEN --censos regionales

                          -- Cuento el numero de electores inscritos en ese censo regional
			     select count(cod_elector) into totalelect from elector where cod_cregional=censo;	
	
	ELSIF (ambito=4) THEN -- censos locales
		-- Cuento el numero de electores inscritos en el censo local		
			     select count(cod_elector) into totalelect from elector where cod_clocal=censo;	
	END IF;
   
        if (operacion=0) then
              -- Aado a los electores de esa votacion los electores inscritos en este censo
              UPDATE votacion SET electores=electores+totalelect where cod_votacion=votacion; 
        elsif (operacion=1)then
              UPDATE votacion SET electores=electores-totalelect where cod_votacion=votacion; 
        end if;
        
	RSP := 'OK, el alta del campo electores en la votacion se ha realizado con xito';

	p_insertaLog('p_insertaelectores',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
		

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al introducir el numero de electores en la votacion' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_insertaelectores',SYSDATE,parametros, RSP);
		commit;
	
END;



/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_inelector
--  OBJETO: Se ejecuta al dar de alta un elector en el censo electoral
--  PARAMETROS DE ENTRADA: region,localidad 
--  PARAMETROS DE SALIDA:   RSP
------------------------------------------------------------------------------------------------------

create or replace PROCEDURE p_inelector (
        region in ELECTOR.COD_CREGIONAL%TYPE,
	localidad in ELECTOR.COD_CLOCAL%TYPE,
        operacion in number,
	RSP OUT Varchar2
) is


  parametros varchar2(250);
  elect integer;
  actualiza number(1);

  vot_record votacion%ROWTYPE;
  vota_record censos_habilitados%rowtype;
  elect_record electores%rowtype;

 -- type vot_pais_record_type is record
 --   (votacion votacion.cod_votacion%type,
 --    estado   votacion.estado%type,
 --    ambito   votacion.ambito%type);
 --  vot_pais_record vot_pais_record_type;

  pais varchar2(2);
  v_nacion integer;
  
  
  -- obtener las votaciones globales que esten en edicion o en pendiente
  cursor votglobal IS
 	Select * from votacion where (ambito=1) and ((estado=1) or (estado=2));
        
  -- cursor para obtener las votaciones de un pais concreto que esten en edicion o pendientes
  cursor votpais(v_nacion integer) IS
	select censos_habilitados.cod_votacion,censos_habilitados.cod_censo from censos_habilitados,votacion where (votacion.cod_votacion= censos_habilitados.cod_votacion) and (censos_habilitados.cod_censo=v_nacion) and (votacion.ambito=2) and ((votacion.estado=1) or (votacion.estado=2));
  
-- cursor para obtener las votaciones de una region en edicion o pendiente
  cursor votregion IS
	select censos_habilitados.cod_votacion,censos_habilitados.cod_censo from censos_habilitados,votacion where (votacion.cod_votacion= censos_habilitados.cod_votacion) and (censos_habilitados.cod_censo=region) and (votacion.ambito=3) and ((votacion.estado=1) or (votacion.estado=2));

-- cursor para obtener las votaciones de una localidad que este en edicion o pendiente
  cursor votlocal IS
	select censos_habilitados.cod_votacion,censos_habilitados.cod_censo from censos_habilitados,votacion where (votacion.cod_votacion= censos_habilitados.cod_votacion) and (censos_habilitados.cod_censo=localidad) and (votacion.ambito=4) and ((votacion.estado=1) or (votacion.estado=2));

  cursor c1 IS
 	Select * from electores;
  BEGIN
	
        -- borro la tabla de votaciones a actualiza ELECTORES
        delete from electores;
        
	parametros := region||','||localidad;

	-- Anadir un elector a las votaciones segun el censo regional y local en el que se inscriba
        


	-- VOTACIONES DE AMBITO GLOBAL
	--******************************


	-- En las votaciones de ambito global siempre hay que actualiza el campo de electores.
	-- Buscar todas aquellas votaciones que no esten abiertas ni finalizadas que sean de tipo global


	   select count(cod_elector) into elect from elector;

           open votglobal; -- Abro el cursor con las votaciones
	    --Por cada votacion actualizo el numero de electores
                    loop
                        fetch votglobal into vot_record;
                        exit when votglobal%notfound;
                            INSERT INTO ELECTORES (cod_votacion) VALUES (vot_record.cod_votacion);
			   -- update votacion set votacion.electores =elect where votacion.cod_votacion =vot_record.cod_votacion;
                           -- DBMS_OUTPUT.PUT_LINE('votacion:'||votacion.COD_VOTACION||', Electores:'||votacion.ELECTORES);
		     end loop;
	   close votglobal; 



	-- VOTACIONES DE AMBITO NACIONAL
	--*******************************

	-- Buscar el pais al que pertenece el censo regional en el que esta inscrito
	   select cod_pais into pais from censo_regional where cod_cregional=region;

	-- Buscar el codigo de censo nacional correspondiente a ese pais
	   select cod_cnacional into v_nacion from censo_nacional where cod_pais=pais;

	-- Obtener el numero de electores de el pais al que pertenece ese censo regional
	--   select count(elector.cod_elector) into elect from censo_nacional,censo_regional,elector where censo_nacional.cod_pais=censo_regional.cod_pais and censo_regional.cod_cregional=elector.cod_cregional and (censo_nacional.cod_cnacional=v_nacion); 

	-- Buscar todas las votaciones que tengan habilitados ese censo nacional y actualizar el campo electores

	   	open votpais(v_nacion); -- Abro el cursor con las votaciones de un pais
			
		--Por cada votacion actualizo el numero de electores
                loop
                     fetch votpais into vota_record;
                     exit when votpais%notfound;
                        INSERT INTO ELECTORES (cod_votacion) VALUES (vota_record.cod_votacion);
			--update votacion set votacion.electores=elect where votacion.cod_votacion=vot_record.cod_votacion;
                       -- DBMS_OUTPUT.PUT_LINE('votacion:'||votacion.COD_VOTACION||', Electores:'||votacion.ELECTORES);
		end loop;
			
		close votpais; 


	-- VOTACIONES DE AMBITO REGIONAL
	--*******************************

	-- Buscar todas las votaciones que tengan habilitados ese censo regional y actualizar el campo electores
	
	-- Obtener el numero de electores inscritos en ese censo regional
	--   select count(elector.cod_elector) into elect from censo_regional,elector where censo_regional.cod_cregional=elector.cod_cregional and censo_regional.cod_cregional=region; 

	open votregion; -- Abro el cursor con las votaciones

	--Por cada votacion actualizo el numero de electores
                loop
                     fetch votregion into vota_record;
                     exit when votregion%notfound;
                        INSERT INTO ELECTORES (cod_votacion) VALUES (vota_record.cod_votacion);
                       --update votacion set votacion.electores=elect where votacion.cod_votacion=vot_record.cod_votacion;
                       -- DBMS_OUTPUT.PUT_LINE('votacion:'||votacion.COD_VOTACION||', Electores:'||votacion.ELECTORES);
		end loop;
			
		close votregion; 


	-- VOTACIONES DE AMBITO LOCAL
	--*******************************

	-- Buscar todas las votaciones que tengan habilitados ese censo local y actualizar el campo electores
	
	-- Obtener el numero de electores inscritos en ese censo local
	 --  select count(elector.cod_elector) into elect from censo_local,elector where censo_local.cod_clocal=elector.cod_clocal and censo_local.cod_clocal=localidad; 

	open votlocal; -- Abro el cursor con las votaciones

	--Por cada votacion actualizo el numero de electores
                loop
                     fetch votlocal into vota_record;
                     exit when votlocal%notfound;
                        INSERT INTO ELECTORES (cod_votacion) VALUES (vota_record.cod_votacion);
                    --update votacion set votacion.electores=elect where votacion.cod_votacion=vot_record.cod_votacion;
                        --DBMS_OUTPUT.PUT_LINE('votacion:'||votacion.COD_VOTACION||', Electores:'||votacion.ELECTORES);
		end loop;
			
		close votlocal; 

        -- en caso de dar de alta un elector
        if (operacion=0) then
          actualiza:=1;
        elsif (operacion=1) then
        -- en caso de dar de baja un elector
          actualiza:=-1;
        end if;
        
        --  Actualizo en la tabla votacion los electores, en esta tabla auxiliar "electores", tengo las votaciones que hay que aadir un elector
        open c1; -- Abro el cursor con las votaciones
	    --Por cada votacion actualizo el numero de electores
                    loop
                        fetch c1 into elect_record;
                        exit when c1%notfound;
                            update votacion set votacion.electores= votacion.electores + actualiza where (votacion.COD_VOTACION=elect_record.COD_VOTACION);
			   -- update votacion set votacion.electores =elect where votacion.cod_votacion =vot_record.cod_votacion;
                           -- DBMS_OUTPUT.PUT_LINE('votacion:'||votacion.COD_VOTACION||', Electores:'||votacion.ELECTORES);
		     end loop;
        close c1; 

        RSP := 'OK, al actualizar el campo electores en la votacion';

	p_insertaLog('p_inelector',SYSDATE,parametros, RSP);
	
	COMMIT;

  EXCEPTION
		

	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al aadir el elector en la votacion' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_aadelector',SYSDATE,parametros, RSP);
		commit;
	
END;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_editaestado
--  OBJETO: Actualizar estado votacion cuando se ejecute el job j_actualizarestado una vez al da
--  PARAMETROS DE ENTRADA:  
--  PARAMETROS DE SALIDA:   
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_editaestado IS

	RSP Varchar2(500);
	parametros varchar2(250);
        vot_record votacion%ROWTYPE;
        cursor votaciones is
		select * from votacion;

		BEGIN 
                              -- Recorremos todas las votaciones para actualizar el estado de cada una
				
                                parametros:=RSP;
				open votaciones; -- Abro el cursor con las votaciones
	    			--Por cada votacion actualizo el estado
                    		loop
                        		fetch votaciones into vot_record;
                        		exit when votaciones%notfound;
						if (vot_record.estado != 4) then
							If(sysdate<vot_record.fecha_publicacion) then
								update votacion set votacion.estado=1 where votacion.cod_votacion=vot_record.cod_votacion;
							elsif (sysdate<vot_record.fecha_inicio) then
								update votacion set votacion.estado=2 where votacion.cod_votacion=vot_record.cod_votacion;
							elsif (sysdate<vot_record.fecha_fin) then
								update votacion set votacion.estado=3 where votacion.cod_votacion=vot_record.cod_votacion;
							else
								update votacion set votacion.estado=4 where votacion.cod_votacion=vot_record.cod_votacion;
							end if;
						end if;
		     		end loop;
	   			close votaciones; 

				-- Actualizo el nmero de votaciones finalizadas por pais y ao
                                 delete est_vot_final_ao;
                                 insert into est_vot_final_ao(select p.cod_pais,extract(year from v.fecha_inicio) ao,count( distinct v.cod_votacion) total  from 
                                 paiscenso p, votacenso v where p.codcenso=v.cod_censo and estado=4 group by p.cod_pais, extract(year from v.fecha_inicio));
				 
				-- actualizo estadistica de valor medio del porcentaje de participacion de las votaciones por censo y ao
				delete est_censo_anyo_porct;
				insert into est_censo_anyo_porct(select cod_censo,extract(year from fecha_inicio) ao ,avg(porcent_participacion) media from votacenso where estado=4 group by cod_censo,extract(year from fecha_inicio));
	
				RSP := 'OK, se ha actualizado el estado de la votacion correctamente';
				p_insertaLog('p_editaestado',SYSDATE,parametros, RSP);

		EXCEPTION	
				WHEN OTHERS THEN
	
				RSP := 'ERROR: Error al actualizar el estado de la votacin: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog('p_editaestado',SYSDATE, parametros, RSP);
				commit;
		END;

/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: j_actulizarestado
--  OBJETO: Job que se ejecuta una vez al da para actualizar el Estado de una votacion
--  PARAMETROS DE ENTRADA:
--  PARAMETROS DE SALIDA:  
------------------------------------------------------------------------------------------------------



BEGIN

  DBMS_SCHEDULER.create_job (
    job_name        => 'j_actualizarestado',
    job_type        => 'PLSQL_BLOCK',
    job_action      => '			
                        BEGIN 
                            p_editaestado;  
                         END;',
    start_date      => SYSDATE,
    repeat_interval => 'freq=daily; byhour=0',
    end_date        => NULL,
    enabled         => TRUE,
    comments        => 'Job para actualizar el estado de una votacion con respecto a la fecha del sistema.');

END;

/





/****************************************************************************************************
*	
*		            CONSULTAS DE LA BASE DE DATOS
*
****************************************************************************************************/





/*-----------------------------------------------------------------------------------------------------
--  NOMBRE: paquete pk_consulta_votacionesPais
--  OBJETO: Consultar las votaciones de un pas con los porcentajes de participacion cuando la votacion est finalizada
--  FUNCIONES:       f_votacionespais
     			  parametros de entrada: codigo_pais
			  return: var_refcursor

--  PROCEDIMIENTOS:  p_listavotpais
			  parametros de entrada: codigo_pais
			  parametros de salida: RSP
------------------------------------------------------------------------------------------------------*/


CREATE OR REPLACE PACKAGE pk_consulta_votacionesPais AS

	-- declaramos el cursor dinamico
	TYPE var_refcursor IS REF CURSOR;

	--declaramos la funcion que devuelve el cursor
	FUNCTION f_votacionespais(codigo_pais in varchar2) RETURN var_refcursor;

	--declaramos el procedimiento que muestra por pantalla el listado
	PROCEDURE p_listavotpais(codigo_pais in varchar2, RSP out varchar2) ;

END;

/

CREATE OR REPLACE
PACKAGE BODY pk_consulta_votacionesPais AS

	FUNCTION f_votacionespais(codigo_pais varchar2) RETURN var_refcursor IS
	var_cursorpackage var_refcursor;

	BEGIN
			OPEN var_cursorpackage FOR
				SELECT * FROM (
						select v.cod_votacion,v.titulo, v.fecha_inicio, v.fecha_fin, v.electores, v.porcent_participacion "participacion", e.nombre "ESTADO", v.optmasvotada "OPCION GANADORA",v.porcent_ganadora "PORCENTAJE G",  v.optmenosvotada "OPCION PERDEDORA", v.porcent_perdedora "PORCENTAJE P" from votacion v, estado e
						where cod_votacion in (
						-- Todas las votaciones relacionadas con el censo de un pas
						  select cod_votacion from censos_habilitados
						  where cod_censo in
						  -- Todos los censos de un pais
						  (select cod_cglobal "Codcenso" from censo_global
						   union
						   select cod_cnacional from censo_nacional where cod_pais=codigo_pais
						   union
						   select cod_cregional from censo_regional where cod_pais=codigo_pais
						   union 
					          select cod_clocal from censo_local where cod_pais=codigo_pais))
						  and v.estado=e.cod_estado order by v.fecha_inicio);

		RETURN var_cursorpackage; -- retornamos el cursor

	END f_votacionespais;



	PROCEDURE p_listavotpais (codigo_pais in varchar2,RSP OUT varchar2) is
		
		parametros varchar2(500);

		--declaramos la variable de tipo ref cursor
		v_refcursor pk_consulta_votacionesPais.var_refcursor;

		-- declaramos la variable registro con los campos que queremos mostrar

		 type vot_pais_record_type is record
 		   (votacio votacion.cod_votacion%type,
		    titul   votacion.titulo%type,
		    f_inici votacion.fecha_inicio%type,
		    f_fi votacion.fecha_fin%type,
		    elec votacion.electores%type,
	   	    v_part votacion.porcent_participacion%type,
		    estad  estado.nombre%type,
		    optma  votacion.optmasvotada%type,
		    porctma  votacion.porcent_ganadora%type,
		    optmeno votacion.optmenosvotada%type,
		    porctmeno votacion.porcent_perdedora%type);
 		

		    registro VOT_PAIS_RECORD_TYPE;
			
		BEGIN
                 parametros := codigo_pais;
			
                  -- Se obtiene el cursor con los datos devueltos por la funcion f_votacionespais del package
                  v_refcursor := pk_consulta_votacionesPais.f_votacionespais(codigo_pais);


		  DBMS_OUTPUT.PUT_LINE('------------ Informe de votaciones por pais ' || TO_CHAR(sysdate, 'DD/MM/YYYY HH24:MI:SS') || ' ---------------');
		  DBMS_OUTPUT.NEW_LINE;
		  DBMS_OUTPUT.PUT_LINE('------------CODIGO PAIS: ' || codigo_pais || ' ---------------');
                  DBMS_OUTPUT.NEW_LINE;

		DBMS_OUTPUT.PUT(rpad('CODIGO',8));
		DBMS_OUTPUT.PUT(rpad('TITULO',50));
		DBMS_OUTPUT.PUT(rpad('FECHA INICIO',15));
		DBMS_OUTPUT.PUT(rpad('FECHA FIN',15));
		DBMS_OUTPUT.PUT(rpad('ELECTORES',12));
                DBMS_OUTPUT.PUT(rpad('ESTADO',15));
		DBMS_OUTPUT.PUT(rpad('% PARTICIPACION',20));
		DBMS_OUTPUT.PUT(rpad('GANADOR',35));
		DBMS_OUTPUT.PUT(rpad('%PARTICIPACION',20));
		DBMS_OUTPUT.PUT(rpad('PERDEDOR',35));
		DBMS_OUTPUT.PUT(rpad('% PARTICIPACION',20));
                DBMS_OUTPUT.NEW_LINE;
		  -- recorremos el cursor hasta que no haya datos
		  LOOP	
		
			-- se obtiene el valor del registro del cursor
			FETCH v_refcursor
			INTO registro;
			exit when v_refcursor%notfound;
			
			--mostramos los datos por pantalla.
				DBMS_OUTPUT.PUT(rpad(registro.votacio, 8) || rpad(registro.titul, 50) || rpad(to_char(registro.f_inici,'DD/MM/YYYY'), 15) || rpad(to_char(registro.f_fi, 'DD/MM/YYYY'), 15));
				DBMS_OUTPUT.PUT(rpad(registro.elec, 12) || rpad(registro.estad,15));
                                IF(registro.estad='finalizada') then
                                  DBMS_OUTPUT.PUT(rpad(registro.v_part ||'%', 20)|| rpad(registro.optma,35));
                                  DBMS_OUTPUT.PUT(rpad(registro.porctma ||'%', 20) || rpad(registro.optmeno, 35) || rpad(registro.porctmeno ||'%',20));
                                ELSE
                                     DBMS_OUTPUT.PUT(rpad('-', 20)|| rpad('-' ,35));
                                     DBMS_OUTPUT.PUT(rpad('-', 20) || rpad('-', 35) || rpad('-',20));
                                END IF;
                                DBMS_OUTPUT.NEW_LINE;
		  END LOOP;

			--cerramos el cursor
			CLOSE v_refcursor;

		
		RSP := 'OK, se han listado las votaciones de un pais correctamente';
		p_insertaLog('p_listavotpais',SYSDATE,parametros, RSP);

		EXCEPTION
	
			WHEN NO_DATA_FOUND THEN
		
				DBMS_OUTPUT.PUT_LINE('NO SE ENCONTRARON DATOS');
		
				RSP := 'ERROR: No existen votaciones para este pas: ' || codigo_pais;

				p_insertaLog('p_listavotpais',SYSDATE,parametros, RSP);	
	
	
			WHEN OTHERS THEN
	
				RSP := 'ERROR: Error al consultar la votaciones del pais: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog('p_listavotpais',SYSDATE,parametros, RSP);	

		END p_listavotpais;

END pk_consulta_votacionesPais;

/

/*-----------------------------------------------------------------------------------------------------
--  NOMBRE: paquete pk_consulta_votmasdif
--  OBJETO: Consultar las 10 votaciones con mayor diferencia porcentual entre la opcion ms votada y menos votada
--  FUNCIONES:       f_votacionesmas
     		     	return: var_refcursor

--  PROCEDIMIENTOS:  p_listavotacionesmas
			parametros de salida: RSP  
------------------------------------------------------------------------------------------------------*/


CREATE OR REPLACE PACKAGE pk_consulta_votmasdif As
	-- declaramos el cursor dinamico
	TYPE var_refcursor IS REF CURSOR;

	--declaramos la funcion que devuelve el cursor
	FUNCTION f_votacionesmas RETURN var_refcursor;

	--declaramos el procedimiento que muestra por pantalla el listado
	PROCEDURE p_listavotacionesmas(RSP out varchar2) ;

END;

/

CREATE OR REPLACE
PACKAGE BODY pk_consulta_votmasdif AS

	FUNCTION f_votacionesmas RETURN var_refcursor IS
	var_cursorpackage var_refcursor;

	BEGIN
			OPEN var_cursorpackage FOR
				select * from (SELECT cod_votacion,titulo,fecha_inicio,fecha_fin,optmasvotada,porcent_ganadora,optmenosvotada,
				porcent_perdedora, porcent_ganadora-porcent_perdedora DIFERENCIA FROM votacion order by diferencia desc)
                                where rownum <=10;

		RETURN var_cursorpackage; -- retornamos el cursor

	END f_votacionesmas;



	PROCEDURE p_listavotacionesmas (RSP OUT varchar2) is
		
		parametros varchar2(500);

		--declaramos la variable de tipo ref cursor
		v_refcursor pk_consulta_votmasdif.var_refcursor;

		-- declaramos la variable registro con los campos que queremos mostrar

		 type vot_record_type is record
 		   (votacio votacion.cod_votacion%type,
		    titul   votacion.titulo%type,
		    f_inici votacion.fecha_inicio%type,
		    f_fi votacion.fecha_fin%type,
		    optma  votacion.optmasvotada%type,
		    porctma  votacion.porcent_ganadora%type,
		    optmeno votacion.optmenosvotada%type,
		    porctmeno votacion.porcent_perdedora%type,
		    diferencia votacion.porcent_ganadora%type);
 		

		    registro VOT_RECORD_TYPE;
			
		BEGIN
                 parametros := ' ';
			
                  -- Se obtiene el cursor con los datos devueltos por la funcion f_votaciones del package
                  v_refcursor := pk_consulta_votmasdif.f_votacionesmas();


		  DBMS_OUTPUT.PUT_LINE('------------ Informe de las 10 votaciones con mayor diferencia porcentual entre la opcion mas votada y la menos votada ' || TO_CHAR(sysdate, 'DD/MM/YYYY HH24:MI:SS') || ' ---------------');
		  DBMS_OUTPUT.NEW_LINE;
		  

		DBMS_OUTPUT.PUT(rpad('CODIGO',8));
		DBMS_OUTPUT.PUT(rpad('TITULO',50));
		DBMS_OUTPUT.PUT(rpad('FECHA INICIO',15));
		DBMS_OUTPUT.PUT(rpad('FECHA FIN',15));
		DBMS_OUTPUT.PUT(rpad('GANADOR',35));
		DBMS_OUTPUT.PUT(rpad('%PARTICIPACION',20));
		DBMS_OUTPUT.PUT(rpad('PERDEDOR',35));
		DBMS_OUTPUT.PUT(rpad('% PARTICIPACION',20));
		DBMS_OUTPUT.PUT(rpad('DIFERENCIA %',20));
                DBMS_OUTPUT.NEW_LINE;
		  -- recorremos el cursor hasta que no haya datos
		  LOOP	
		
			-- se obtiene el valor del registro del cursor
			FETCH v_refcursor
			INTO registro;
			exit when v_refcursor%notfound;
			
			--mostramos los datos por pantalla.
				DBMS_OUTPUT.PUT(rpad(registro.votacio, 8) || rpad(registro.titul, 50) || rpad(to_char(registro.f_inici,'DD/MM/YYYY'), 15) || rpad(to_char(registro.f_fi, 'DD/MM/YYYY'), 15));
                                DBMS_OUTPUT.PUT(rpad(registro.optma,35));
                                DBMS_OUTPUT.PUT(rpad(registro.porctma ||'%', 20) || rpad(registro.optmeno, 35) || rpad(registro.porctmeno ||'%',20));
                                DBMS_OUTPUT.PUT(rpad(registro.diferencia ||'%',20));
				DBMS_OUTPUT.NEW_LINE;
		  END LOOP;

			--cerramos el cursor
			CLOSE v_refcursor;

		
		RSP := 'OK, se han listado las 10 votaciones con mayor diferencia porcentual entre la opcion mas votada y menos votada';
		p_insertaLog('p_listavotacionesmas',SYSDATE,parametros, RSP);

		EXCEPTION
	
			WHEN OTHERS THEN
	
				RSP := 'ERROR: Error al listar las votaciones: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog('p_listavotacionesmas',SYSDATE,parametros, RSP);	

		END p_listavotacionesmas;

END pk_consulta_votmasdif;

/

/*-----------------------------------------------------------------------------------------------------
--  NOMBRE: paquete pk_consulta_votmenosdif
--  OBJETO: Consultar las 10 votaciones con menor diferencia porcentual entre la opcion ms votada y menos votada
--  FUNCIONES:       f_votacionesmenos
     			return: var_refcursor  

--  PROCEDIMIENTOS:  p_listavotacionesmenos
			parametros de salida: RSP    
------------------------------------------------------------------------------------------------------*/


CREATE OR REPLACE PACKAGE pk_consulta_votmenosdif As
	-- declaramos el cursor dinamico
	TYPE var_refcursor IS REF CURSOR;

	--declaramos la funcion que devuelve el cursor
	FUNCTION f_votacionesmenos RETURN var_refcursor;

	--declaramos el procedimiento que muestra por pantalla el listado
	PROCEDURE p_listavotacionesmenos(RSP out varchar2) ;

END;

/

CREATE OR REPLACE
PACKAGE BODY pk_consulta_votmenosdif AS

	FUNCTION f_votacionesmenos RETURN var_refcursor IS
	var_cursorpackage var_refcursor;

	BEGIN
			OPEN var_cursorpackage FOR
				select * from (SELECT cod_votacion,titulo,fecha_inicio,fecha_fin,optmasvotada,porcent_ganadora,optmenosvotada,
				porcent_perdedora, porcent_ganadora-porcent_perdedora DIFERENCIA FROM votacion order by diferencia)
                                where rownum<=10;
		RETURN var_cursorpackage; -- retornamos el cursor

	END f_votacionesmenos;



	PROCEDURE p_listavotacionesmenos (RSP OUT varchar2) is
		
		parametros varchar2(500);

		--declaramos la variable de tipo ref cursor
		v_refcursor pk_consulta_votmasdif.var_refcursor;

		-- declaramos la variable registro con los campos que queremos mostrar

		 type vot_record_type is record
 		   (votacio votacion.cod_votacion%type,
		    titul   votacion.titulo%type,
		    f_inici votacion.fecha_inicio%type,
		    f_fi votacion.fecha_fin%type,
		    optma  votacion.optmasvotada%type,
		    porctma  votacion.porcent_ganadora%type,
		    optmeno votacion.optmenosvotada%type,
		    porctmeno votacion.porcent_perdedora%type,
		    diferencia votacion.porcent_ganadora%type);
 		

		    registro VOT_RECORD_TYPE;
			
		BEGIN
                 parametros := ' ';
			
                  -- Se obtiene el cursor con los datos devueltos por la funcion f_votaciones del package
                  v_refcursor := pk_consulta_votmenosdif.f_votacionesmenos();


		  DBMS_OUTPUT.PUT_LINE('------------ Informe de las 10 votaciones con menor diferencia porcentual entre la opcion mas votada y la menos votada ' || TO_CHAR(sysdate, 'DD/MM/YYYY HH24:MI:SS') || ' ---------------');
		  DBMS_OUTPUT.NEW_LINE;
		  

		DBMS_OUTPUT.PUT(rpad('CODIGO',8));
		DBMS_OUTPUT.PUT(rpad('TITULO',50));
		DBMS_OUTPUT.PUT(rpad('FECHA INICIO',15));
		DBMS_OUTPUT.PUT(rpad('FECHA FIN',15));
		DBMS_OUTPUT.PUT(rpad('GANADOR',35));
		DBMS_OUTPUT.PUT(rpad('%PARTICIPACION',20));
		DBMS_OUTPUT.PUT(rpad('PERDEDOR',35));
		DBMS_OUTPUT.PUT(rpad('% PARTICIPACION',20));
		DBMS_OUTPUT.PUT(rpad('DIFERENCIA %',20));
                DBMS_OUTPUT.NEW_LINE;
		  -- recorremos el cursor hasta que no haya datos
		  LOOP	
		
			-- se obtiene el valor del registro del cursor
			FETCH v_refcursor
			INTO registro;
			exit when v_refcursor%notfound;
			
			--mostramos los datos por pantalla.
				DBMS_OUTPUT.PUT(rpad(registro.votacio, 8) || rpad(registro.titul, 50) || rpad(to_char(registro.f_inici,'DD/MM/YYYY'), 15) || rpad(to_char(registro.f_fi, 'DD/MM/YYYY'), 15));
                                DBMS_OUTPUT.PUT(rpad(registro.optma,35));
                                DBMS_OUTPUT.PUT(rpad(registro.porctma ||'%', 20) || rpad(registro.optmeno, 35) || rpad(registro.porctmeno ||'%',20));
                                DBMS_OUTPUT.PUT(rpad(registro.diferencia ||'%',20));
				DBMS_OUTPUT.NEW_LINE;
		  END LOOP;

			--cerramos el cursor
			CLOSE v_refcursor;

		
		RSP := 'OK, se han listado las 10 votaciones con menor diferencia porcentual entre la opcion mas votada y menos votada';
		p_insertaLog('p_listavotacionesmas',SYSDATE,parametros, RSP);

		EXCEPTION
	
			WHEN OTHERS THEN
	
				RSP := 'ERROR: Error al listar las votaciones: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog('p_listavotacionesmas',SYSDATE,parametros, RSP);	

		END p_listavotacionesmenos;

END pk_consulta_votmenosdif;

/

/*-----------------------------------------------------------------------------------------------------
--  NOMBRE: paquete pk_consulta_censos
--  OBJETO: Consultar los censos en los que est inscrito un ciudadano
--  FUNCIONES:       f_censos
     			  parametros de entrada: codigo_ciudadano

--  PROCEDIMIENTOS:  p_listacensos
			  parametros de entrada: codigo_ciudadano
			  parametros de salida: RSP
------------------------------------------------------------------------------------------------------*/


CREATE OR REPLACE PACKAGE pk_consulta_censos AS

	-- declaramos el cursor dinamico
	TYPE var_refcursor IS REF CURSOR;

	--declaramos la funcion que devuelve el cursor
	FUNCTION f_censos(codigo_ciudadano in varchar2) RETURN var_refcursor;

	--declaramos el procedimiento que muestra por pantalla el listado
	PROCEDURE p_listacensos(codigo_ciudadano in varchar2, RSP out varchar2) ;

END;

/

CREATE OR REPLACE
PACKAGE BODY pk_consulta_censos AS

	FUNCTION f_censos(codigo_ciudadano varchar2) RETURN var_refcursor IS
	var_cursorpackage var_refcursor;

	BEGIN
			OPEN var_cursorpackage FOR
				
				select e.cod_elector,cg.cod_cglobal,cg.nombre,cn.cod_cnacional,cn.nom_pais,cr.cod_cregional,cr.nom_region,e.cod_clocal,cl.nom_localidad from 
				censo_regional cr, censo_nacional cn, censo_global cg, elector e,censo_local cl
				where cr.cod_pais=cn.cod_pais and cg.cod_cglobal = cn.cod_cglobal and e.cod_cregional=cr.cod_cregional and e.cod_clocal=cl.cod_clocal
				and e.cod_elector=codigo_ciudadano;

			RETURN var_cursorpackage; -- retornamos el cursor

	END f_censos;



	PROCEDURE p_listacensos (codigo_ciudadano in varchar2,RSP OUT varchar2) is
		
		parametros varchar2(500);

		--declaramos la variable de tipo ref cursor
		v_refcursor pk_consulta_censos.var_refcursor;

		-- declaramos la variable registro con los campos que queremos mostrar

		 type censo_record_type is record
 		   (elect elector.cod_elector%type,
		    cglobal censo_global.cod_cglobal%type,
		    cgnombre censo_global.nombre%type,
		    cnacional censo_nacional.cod_cnacional%type,
		    cnnombre censo_nacional.nom_pais%type,
		    cregional censo_regional.cod_cregional%type,
		    crnombre censo_regional.nom_region%type,
		    clocal censo_local.cod_clocal%type,
		    clnombre censo_local.nom_localidad%type);

		    registro CENSO_RECORD_TYPE;
			
		BEGIN
                 parametros := codigo_ciudadano;
			
                  -- Se obtiene el cursor con los datos devueltos por la funcion f_censos del package
                  v_refcursor := pk_consulta_censos.f_censos(codigo_ciudadano);


		  DBMS_OUTPUT.PUT_LINE('------------ Informe de censos en los que esta inscrito un ciudadano ' || TO_CHAR(sysdate, 'DD/MM/YYYY HH24:MI:SS') || ' ---------------');
		  DBMS_OUTPUT.NEW_LINE;
		  DBMS_OUTPUT.PUT_LINE('------------CODIGO CIUDADANO: ' || codigo_ciudadano || ' ---------------');
                  DBMS_OUTPUT.NEW_LINE;

		DBMS_OUTPUT.PUT(rpad('CODIGO CIUDADANO',20));
		DBMS_OUTPUT.PUT(rpad('CENSO GLOBAL',20));
		DBMS_OUTPUT.PUT(rpad('NOMBRE',20));
		DBMS_OUTPUT.PUT(rpad('CENSO NACIONAL',20));
		DBMS_OUTPUT.PUT(rpad('NOMBRE',20));
                DBMS_OUTPUT.PUT(rpad('CENSO REGIONAL',20));
		DBMS_OUTPUT.PUT(rpad('NOMBRE',20));
		DBMS_OUTPUT.PUT(rpad('CENSO LOCAL',20));
		DBMS_OUTPUT.PUT(rpad('NOMBRE',20));
                DBMS_OUTPUT.NEW_LINE;
		  -- recorremos el cursor hasta que no haya datos
		  LOOP	
		
			-- se obtiene el valor del registro del cursor
			FETCH v_refcursor
			INTO registro;
			exit when v_refcursor%notfound;
			
			--mostramos los datos por pantalla.
				DBMS_OUTPUT.PUT(rpad(registro.elect, 20) || rpad(registro.cglobal, 20) || rpad(registro.cgnombre, 20) || rpad(registro.cnacional, 20));
				DBMS_OUTPUT.PUT(rpad(registro.cnnombre, 20) || rpad(registro.cregional,20) || rpad(registro.crnombre,20) || rpad(registro.clocal,20));
				DBMS_OUTPUT.PUT(rpad(registro.clnombre, 20));
                                DBMS_OUTPUT.NEW_LINE;
		  END LOOP;

			--cerramos el cursor
			CLOSE v_refcursor;

		
		RSP := 'OK, se han listado los censos en los que esta inscrito un ciudadano correctamente';
		p_insertaLog('p_listacensos',SYSDATE,parametros, RSP);

		EXCEPTION
	
			WHEN NO_DATA_FOUND THEN
		
				DBMS_OUTPUT.PUT_LINE('NO SE ENCONTRARON DATOS');
		
				RSP := 'ERROR: No existen censos para este ciudadano: ' || codigo_ciudadano;

				p_insertaLog('p_listacensos',SYSDATE,parametros, RSP);	
	
	
			WHEN OTHERS THEN
	
				RSP := 'ERROR: Error al consultar los censos de un ciudadano: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog('p_listacensos',SYSDATE,parametros, RSP);	

		END p_listacensos;

END pk_consulta_censos;

/



/*-----------------------------------------------------------------------------------------------------
--  NOMBRE: paquete pk_consulta_participacion
--  OBJETO: Consultar las votaciones en las que ha participado un ciudadano
--  FUNCIONES:       f_votaciones
     			  parametros de entrada: codigo_ciudadano

--  PROCEDIMIENTOS:  p_listavotaciones
			  parametros de entrada: codigo_ciudadano
			  parametros de salida: RSP
------------------------------------------------------------------------------------------------------*/


CREATE OR REPLACE PACKAGE pk_consulta_participacion AS

	-- declaramos el cursor dinamico
	TYPE var_refcursor IS REF CURSOR;

	--declaramos la funcion que devuelve los censos en los que est inscrito un ciudadano
	FUNCTION f_censos(codigo_ciudadano in varchar2) RETURN var_refcursor;

	--declaramos la funcion que devuelve el cursor con las votaciones de un ciudadano
	FUNCTION f_votaciones(codigo_ciudadano in varchar2) RETURN var_refcursor;


	--declaramos el procedimiento que muestra por pantalla el listado
	PROCEDURE p_listavotaciones(codigo_ciudadano in varchar2, RSP out varchar2) ;

END;

/

create or replace PACKAGE BODY pk_consulta_participacion AS


	FUNCTION f_censos(codigo_ciudadano varchar2) RETURN var_refcursor IS
	var_cursorpackage var_refcursor;

	BEGIN
			OPEN var_cursorpackage FOR
				
				select cn.cod_cnacional,cr.cod_cregional,e.cod_clocal from 
				censo_regional cr, censo_nacional cn, censo_global cg, elector e,censo_local cl
				where cr.cod_pais=cn.cod_pais and cg.cod_cglobal = cn.cod_cglobal and e.cod_cregional=cr.cod_cregional and e.cod_clocal=cl.cod_clocal
				and e.cod_elector=codigo_ciudadano;

			RETURN var_cursorpackage; -- retornamos el cursor

	END f_censos;

	
	FUNCTION f_votaciones(codigo_ciudadano varchar2) RETURN var_refcursor IS
	
	var_cursorpackage var_refcursor;

	--declaramos la variable de tipo ref cursor
	v_refcursor pk_consulta_censos.var_refcursor;
        
	v_cnacional censo_nacional.cod_cnacional%type;
        v_cregional censo_regional.cod_cregional%type;
        v_clocal elector.cod_clocal%type;
        

	BEGIN
			v_refcursor := pk_consulta_participacion.f_censos(codigo_ciudadano);
					
			FETCH v_refcursor
			INTO v_cnacional,v_cregional,v_clocal;

			OPEN var_cursorpackage FOR

				SELECT CH.COD_VOTACION, V.TITULO,V.FECHA_INICIO,V.FECHA_FIN, 'NO PARTICIPA' PARTICIPACION FROM CENSOS_HABILITADOS CH, VOTACION V
				WHERE CH.COD_VOTACION=V.COD_VOTACION AND COD_CENSO=v_cnacional
				UNION
				SELECT CH.COD_VOTACION, V.TITULO,V.FECHA_INICIO,V.FECHA_FIN, 'NO PARTICIPA' PARTICIPACION FROM CENSOS_HABILITADOS CH, VOTACION V
				WHERE CH.COD_VOTACION=V.COD_VOTACION AND COD_CENSO=v_cregional
				UNION
				SELECT CH.COD_VOTACION, V.TITULO,V.FECHA_INICIO,V.FECHA_FIN, 'NO PARTICIPA' PARTICIPACION FROM CENSOS_HABILITADOS CH, VOTACION V
				WHERE CH.COD_VOTACION=V.COD_VOTACION AND COD_CENSO=v_clocal

				MINUS

				SELECT VT.COD_VOTACION,V.TITULO,V.FECHA_INICIO,V.FECHA_FIN, 'NO PARTICIPA' PARTICIPACION FROM VOTAR VT, VOTACION V WHERE COD_VOTANTE=codigo_ciudadano AND VT.COD_VOTACION=V.COD_VOTACION

				UNION

				SELECT VT.COD_VOTACION,V.TITULO,V.FECHA_INICIO,V.FECHA_FIN, 'PARTICIPA' PARTICIPACION  FROM VOTAR VT, VOTACION V WHERE COD_VOTANTE=codigo_ciudadano AND VT.COD_VOTACION=V.COD_VOTACION;
				
                        CLOSE v_refcursor;
			RETURN var_cursorpackage; -- retornamos el cursor

	END f_votaciones;



	PROCEDURE p_listavotaciones (codigo_ciudadano in varchar2,RSP OUT varchar2) is
		
		parametros varchar2(500);

		--declaramos la variable de tipo ref cursor
		v_refcursor pk_consulta_censos.var_refcursor;

		-- declaramos la variable registro con los campos que queremos mostrar

		 type vot_record_type is record
 		   (vot  votacion.cod_votacion%type,
		    titul votacion.titulo%type,
		    f_ini votacion.fecha_inicio%type,
		    f_fin votacion.fecha_fin%type,
		    participa varchar(20));

		    registro VOT_RECORD_TYPE;
			
		BEGIN

                 parametros := codigo_ciudadano;
			
                  -- Se obtiene el cursor con los datos devueltos por la funcion f_censos del package
                  v_refcursor := pk_consulta_participacion.f_votaciones(codigo_ciudadano);


		  DBMS_OUTPUT.PUT_LINE('------------ Informe de votaciones en las que participa un  ciudadano ' || TO_CHAR(sysdate, 'DD/MM/YYYY HH24:MI:SS') || ' ---------------');
		  DBMS_OUTPUT.NEW_LINE;
		  DBMS_OUTPUT.PUT_LINE('------------CODIGO CIUDADANO: ' || codigo_ciudadano || ' ---------------');
                  DBMS_OUTPUT.NEW_LINE;

		DBMS_OUTPUT.PUT(rpad('CODIGO VOTACION',20));
		DBMS_OUTPUT.PUT(rpad('TITULO',50));
		DBMS_OUTPUT.PUT(rpad('FECHA INICIO',20));
		DBMS_OUTPUT.PUT(rpad('FECHA FIN',20));
		DBMS_OUTPUT.PUT(rpad('PARTICIPACION',20));
                DBMS_OUTPUT.NEW_LINE;
		  -- recorremos el cursor hasta que no haya datos
		  LOOP	
		
			-- se obtiene el valor del registro del cursor
			FETCH v_refcursor
			INTO registro;
			exit when v_refcursor%notfound;
			
			--mostramos los datos por pantalla.
				DBMS_OUTPUT.PUT(rpad(registro.vot, 20) || rpad(registro.titul, 50) || rpad(to_char(registro.f_ini,'DD/MM/YYYY'), 20) || rpad(to_char(registro.f_fin,'DD/MM/YYYY'), 20));
                                DBMS_OUTPUT.PUT(rpad(registro.participa, 20));
                                DBMS_OUTPUT.NEW_LINE;
		  END LOOP;

			--cerramos el cursor
			CLOSE v_refcursor;

		
		RSP := 'OK, se han listado las votaciones en las que puede participar un ciudadano correctamente';
		p_insertaLog('p_listavotaciones',SYSDATE,parametros, RSP);

		EXCEPTION
	
			WHEN NO_DATA_FOUND THEN
		
				DBMS_OUTPUT.PUT_LINE('NO SE ENCONTRARON DATOS');
		
				RSP := 'ERROR: No existen votaciones para este ciudadano: ' || codigo_ciudadano;

				p_insertaLog('p_listavotaciones',SYSDATE,parametros, RSP);	
	
	
			WHEN OTHERS THEN
	
				RSP := 'ERROR: Error al consultar las votaciones de un ciudadano: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog('p_listavotaciones',SYSDATE,parametros, RSP);	

		END p_listavotaciones;

END pk_consulta_participacion;
/

-----------------------------------------------------------------------------------------------------
--  NOMBRE: p_rellenaestadisticas
--  OBJETO: actualizar las estadisticas cada vez que un ciudadano emite un voto
--  PARAMETROS DE ENTRADA:
--  PARAMETROS DE SALIDA:  
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE
PROCEDURE p_rellenaEstadisticas IS

estanyo varchar2(4);
parametros VARCHAR2(500);
RSP varchar2(500);

-- Cursor con los distintos aos en los que se han celebrado elecciones
Cursor Anyosvota is select distinct extract(year from fecha_inicio) from votacion;



BEGIN

 	parametros := '';

	-- Borramos todas las tablas de etadsticas para refrescar los datos	
	delete est_vot_mas_parti_anyo;
        delete est_loc_mas_votos_anyo;
	delete est_ciudadanos_sinvotos_anyo;
        delete est_historico_votaciones;
        delete est_historico_mas;
	delete est_historico_menos;
 	delete est_votac_pais_persona;


	-- actualizo estadistica votacion con mas participacion en un ao concreto

	open anyosvota;
	 
	 Fetch anyosvota into estanyo;	

 	--Recorremos todos los aos 
	 WHILE anyosvota%FOUND LOOP
	
	 
	 insert into est_vot_mas_parti_anyo(

	-- rellenar estadisticas votacion con mayor participacion por ao
	 select cod_votacion, extract(year from fecha_inicio), porcent_participacion from votacion
	 where extract(year from fecha_inicio)=estanyo and  porcent_participacion=
	 (select  max(porcent_participacion) from votacion v where extract(year from fecha_inicio)=estanyo));
	 
	 
	-- rellenar estadisticas localidad europea con mas votos emitidos por sus ciudadanos
	insert into est_loc_mas_votos_anyo( select * from votalocal where anyo=estanyo and totalvotos=
	(select max(totalvotos) from votalocal where anyo=estanyo group by anyo));


	
	 Fetch anyosvota into estanyo;
	 
	 END LOOP;
	
	 -- hay que tener en cuenta que un ciudadano en un mismo ao puede votar a una votacion autonomica por ejemplo y no votar a las nacionales. En este caso
	 --  se interpreta que el ciudadano ha votado en ese ao,aunque en las nacionales no haya votado.

	 -- rellenar estadisticas porcentaje de ciudadanos que nunca han votado
	 insert into est_ciudadanos_sinvotos_anyo (select extract(year from v.fecha_inicio) ao,(count(distinct(ce.cod_elector))-count(distinct(vt.cod_votante)))/count(distinct(ce.cod_elector))*100 novotan 
	 from votacion v,votar vt,censos_habilitados ch, censoselector ce where v.cod_votacion=ch.cod_votacion and v.cod_votacion=vt.cod_votacion(+) and (ch.cod_censo=ce.cod_clocal or ch.cod_censo=ce.cod_cregional or ch.cod_censo=ce.cod_cnacional)
	 group by extract(year from fecha_inicio));

	 -- rellenar estadisticas votacion que historicamente ha tenido mayor participacion
	 insert into est_historico_votaciones(select v.cod_votacion, v.porcent_participacion from votacion v where porcent_participacion=
	 (select max(porcent_participacion) from votacion));

	 -- rellenar estadisticas votacion que historicamente ha tenido mayor diferencia porcentual entre la opcion ms votada y la menos votada
	 insert into est_historico_mas (select cod_votacion, porcent_ganadora from votacion where (porcent_ganadora-porcent_perdedora)=
	 (select max(v.porcent_ganadora-v.porcent_perdedora) diferencia from votacion v));	
 
	 -- rellenar estadisticas votacion que historicamente ha tenido menor diferencia porcentual entre la opcion ms votada y la menos votada
	 insert into est_historico_menos (select cod_votacion, porcent_ganadora from votacion where (porcent_ganadora-porcent_perdedora)=
	 (select min(v.porcent_ganadora-v.porcent_perdedora) diferencia from votacion v));	

	 -- rellenar estadistica numero maximo de votaciones de un mismo ciudadano y su pais
	 insert into est_votac_pais_persona (select c.cod_elector,c.cod_pais,m.tot as maximovotaciones from censoselector c, maxvotacion m 
	 where c.cod_elector=m.votante);	 

	 RSP := 'OK, se han actualizado las estadisticas correctamente';
     	 p_insertaLog('p_rellenaestadisticas',SYSDATE, parametros, RSP);

	commit;
	 
EXCEPTION
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al rellenar las estadisticas: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog('p_rellenaestadisticas',SYSDATE, parametros, RSP);
		commit;
END;
/