-- Creacion de Tablas
--********************************************************************************************
--                         TABLA ESTADO
--********************************************************************************************

CREATE TABLE ESTADO(
  cod_estado NUMBER(2),
  nombre VARCHAR2(10) NOT NULL,
  descripcion VARCHAR2 (100),
  CONSTRAINT pk_estado PRIMARY KEY (cod_estado),
  CONSTRAINT uk_estado UNIQUE (nombre),
  CONSTRAINT ch_estado CHECK (LOWER(nombre) IN ('edicion','pendiente','abierta','finalizada'))
  );
/
CREATE SEQUENCE SEQ_ESTADO
  INCREMENT BY 1
  START WITH 1
  NOCYCLE
  NOCACHE;
/
CREATE OR REPLACE
 TRIGGER INSERT_ESTADO
 BEFORE INSERT ON ESTADO
 FOR EACH ROW
  BEGIN 
    Select SEQ_ESTADO.NEXTVAL INTO :NEW.cod_estado FROM DUAL;
  END INSERT_ESTADO;
/

--********************************************************************************************
--                         TABLA AMBITO
--********************************************************************************************

CREATE TABLE AMBITO(
  cod_ambito NUMBER(1),
  descripcion VARCHAR2 (10) NOT NULL,
  CONSTRAINT pk_ambito PRIMARY KEY (cod_ambito),
  CONSTRAINT uk_ambito UNIQUE (descripcion), 
  CONSTRAINT ch_ambito CHECK (LOWER(descripcion) IN ('global','nacional','regional','local'))
  );

/
CREATE SEQUENCE SEQ_AMBITO
  INCREMENT BY 1
  START WITH 1
  NOCYCLE
  NOCACHE;
/
CREATE OR REPLACE
 TRIGGER INSERT_AMBITO
 BEFORE INSERT ON AMBITO
 FOR EACH ROW
  BEGIN 
    Select SEQ_AMBITO.NEXTVAL INTO :NEW.cod_ambito FROM DUAL;
  END INSERT_AMBITO;

/
--********************************************************************************************
--                         TABLA VOTACION
--********************************************************************************************

CREATE TABLE VOTACION(
  cod_votacion INTEGER,
  titulo VARCHAR2(50) NOT NULL,
  descripcion VARCHAR2 (100),
  fecha_publicacion DATE NOT NULL,
  fecha_inicio DATE NOT NULL,
  fecha_fin DATE NOT NULL,
  estado NUMBER(1) NOT NULL,
  direccion_web VARCHAR2(50),
  electores INTEGER,
  votos_totales INTEGER,
  porcent_participacion NUMBER(5,2) default 0,
  optmasvotada VARCHAR2 (50) default '-',
  porcent_ganadora NUMBER(5,2) default 0,
  optmenosvotada VARCHAR2 (50) default '-',
  porcent_perdedora NUMBER(5,2) default 0,
  ambito NUMBER(1) NOT NULL,
  presidente VARCHAR2(50) NOT NULL,
  CONSTRAINT pk_votacion PRIMARY KEY (cod_votacion),
  CONSTRAINT uk_votacion UNIQUE (titulo, fecha_inicio), 
  CONSTRAINT fk_estado FOREIGN KEY (estado) REFERENCES ESTADO(cod_estado),
  CONSTRAINT fk_ambito FOREIGN KEY (ambito) REFERENCES AMBITO (cod_ambito),
  CONSTRAINT ck_publicacion CHECK ((fecha_inicio-fecha_publicacion)>7),
  CONSTRAINT ck_fecha_inicio CHECK (fecha_inicio < fecha_fin)
  );
/
CREATE SEQUENCE SEQ_VOTACION
  INCREMENT BY 1
  START WITH 1
  NOCYCLE
  NOCACHE;
/
CREATE OR REPLACE
 TRIGGER INSERT_VOTACION
 BEFORE INSERT ON VOTACION
 FOR EACH ROW
  BEGIN 
    Select SEQ_VOTACION.NEXTVAL INTO :NEW.cod_votacion FROM DUAL;
  END INSERT_VOTACION;
/

create or replace TRIGGER CENSO_GLOBAL AFTER INSERT OR UPDATE OF AMBITO ON VOTACION 
FOR EACH ROW 
BEGIN
     IF UPDATING THEN
          if(:new.ambito != :old.ambito) then
            DELETE  FROM CENSOS_HABILITADOS where cod_votacion=:old.cod_votacion;
          end if;
     END IF;
     IF (:new.ambito=1)THEN
        FOR i in 2..28 loop
              INSERT INTO CENSOS_HABILITADOS(cod_votacion,cod_censo) VALUES (:NEW.cod_votacion,i);
        END LOOP;
    END IF;
END CENSO_GLOBAL;
/
  
--********************************************************************************************
--                         TABLA CENSO
--********************************************************************************************

CREATE TABLE CENSO(
  cod_censo INTEGER,
  tipo_censo NUMBER (1) NOT NULL,
  CONSTRAINT pk_censo PRIMARY KEY (cod_censo),
  CONSTRAINT fk_censo FOREIGN KEY (tipo_censo) REFERENCES AMBITO (cod_ambito)
  );

/

CREATE SEQUENCE SEQ_CENSO
  INCREMENT BY 1
  START WITH 1
  NOCYCLE
  NOCACHE;
/

--********************************************************************************************
--                         TABLA CENSOS_HABILITADOS
--********************************************************************************************
/
CREATE TABLE CENSOS_HABILITADOS(
  cod_votacion INTEGER,
  cod_censo INTEGER,
  CONSTRAINT pk_censohabilitado PRIMARY KEY (cod_votacion,cod_censo),
  CONSTRAINT fk_votacion FOREIGN KEY (cod_votacion) REFERENCES VOTACION (cod_votacion) ON DELETE CASCADE,
  CONSTRAINT fk_censoh FOREIGN KEY (cod_censo) REFERENCES CENSO (cod_censo)
  );
/

--********************************************************************************************
--                         TABLA CENSO_GLOBAL
--********************************************************************************************

CREATE TABLE CENSO_GLOBAL(
  cod_cglobal INTEGER,
  nombre VARCHAR2(50),
  CONSTRAINT uk_cglobal UNIQUE (nombre), 
  CONSTRAINT pk_cglobal PRIMARY KEY (cod_cglobal),
  CONSTRAINT fk_cglobal FOREIGN KEY (cod_cglobal) REFERENCES CENSO (cod_censo) ON DELETE CASCADE
  );
/


CREATE OR REPLACE
 TRIGGER INSERT_CENSO_GLOBAL
 BEFORE INSERT ON CENSO_GLOBAL
 FOR EACH ROW
  
  DECLARE
  
  codigo integer;
  
  BEGIN 
    Select SEQ_CENSO.NEXTVAL INTO codigo FROM DUAL;
    :NEW.cod_cglobal:=codigo;
   
-- Tipo censo 1 corresponde al ambito global

    INSERT INTO CENSO (cod_censo,tipo_censo) VALUES (codigo,1);
    

  END INSERT_CENSO_GLOBAL;
/

--********************************************************************************************
--                         TABLA CENSO_NACIONAL
--********************************************************************************************

CREATE TABLE CENSO_NACIONAL(
  cod_cnacional INTEGER,
  cod_cglobal INTEGER NOT NULL,
  cod_pais VARCHAR2(2) NOT NULL,
  nom_pais VARCHAR2(50),
  CONSTRAINT pk_cnacional PRIMARY KEY (cod_cnacional),
  CONSTRAINT fk_cnacional FOREIGN KEY (cod_cnacional) REFERENCES CENSO (cod_censo) ON DELETE CASCADE,
  CONSTRAINT fk_cnacional2 FOREIGN KEY (cod_cglobal) REFERENCES CENSO_GLOBAL (cod_cglobal) ON DELETE CASCADE,
  CONSTRAINT uk_pais UNIQUE (cod_pais),
  CONSTRAINT ck_pais CHECK (UPPER(cod_pais) in ('AT','BE','BG','CY','CZ','DK','EE','FI','FR','DE','GR','HU','IE','IT','LV','LT','LU','MT','NL','PL','PT','RO','SK','SI','ES','SE','GB'))
  );


/
CREATE OR REPLACE
 TRIGGER INSERT_CENSO_NACIONAL
 BEFORE INSERT ON CENSO_NACIONAL
 FOR EACH ROW
  
  DECLARE
  
  codigo integer;
  
  BEGIN 
    Select SEQ_CENSO.NEXTVAL INTO codigo FROM DUAL;
    :NEW.cod_cnacional:=codigo;
   
-- Tipo censo 2 corresponde al ambito nacional

    INSERT INTO CENSO (cod_censo,tipo_censo) VALUES (codigo,2);
    

  END INSERT_CENSO_NACIONAL;
/


--********************************************************************************************
--                         TABLA CENSO_REGIONAL
--********************************************************************************************

CREATE TABLE CENSO_REGIONAL(
  cod_cregional INTEGER,
  cod_pais  VARCHAR2(2) NOT NULL,
  cod_region VARCHAR2(10) NOT NULL,
  nom_region VARCHAR(50),
  CONSTRAINT pk_cregion PRIMARY KEY (cod_cregional),
  CONSTRAINT fk_cregion FOREIGN KEY (cod_cregional) REFERENCES CENSO (cod_censo) ON DELETE CASCADE,
  CONSTRAINT fk_cregion2 FOREIGN KEY (cod_pais) REFERENCES CENSO_NACIONAL (cod_pais) ON DELETE CASCADE,
  CONSTRAINT uk_region UNIQUE (cod_pais,cod_region)
  );

/
CREATE OR REPLACE
 TRIGGER INSERT_CENSO_REGIONAL
 BEFORE INSERT ON CENSO_REGIONAL
 FOR EACH ROW
  
  DECLARE
  
  codigo integer;
  
  BEGIN 
    Select SEQ_CENSO.NEXTVAL INTO codigo FROM DUAL;
    :NEW.cod_cregional:=codigo;
   
-- Tipo censo 3 corresponde al ambito regional

    INSERT INTO CENSO (cod_censo,tipo_censo) VALUES (codigo,3);
    

  END INSERT_CENSO_REGIONAL;
/

--********************************************************************************************
--                         TABLA CENSO_LOCAL
--********************************************************************************************

CREATE TABLE CENSO_LOCAL(
  cod_clocal INTEGER,
  cod_pais  VARCHAR2(2) NOT NULL,
  cod_region VARCHAR2(10) NOT NULL,
  cod_localidad VARCHAR2(10) NOT NULL,
  nom_localidad VARCHAR(50),
  CONSTRAINT pk_clocalidad PRIMARY KEY (cod_clocal),
  CONSTRAINT fk_clocalidad FOREIGN KEY (cod_clocal) REFERENCES CENSO (cod_censo) ON DELETE CASCADE,
  CONSTRAINT fk_clocalidad2 FOREIGN KEY (cod_pais,cod_region) REFERENCES CENSO_REGIONAL (cod_pais,cod_region) ON DELETE CASCADE,
  CONSTRAINT uk_localidad UNIQUE (cod_pais,cod_region,cod_localidad)
  );
  

/
CREATE OR REPLACE
 TRIGGER INSERT_CENSO_LOCAL
 BEFORE INSERT ON CENSO_LOCAL
 FOR EACH ROW
  
  DECLARE
  
  codigo integer;
  
  BEGIN 
    Select SEQ_CENSO.NEXTVAL INTO codigo FROM DUAL;
    :NEW.cod_clocal:=codigo;
   
-- Tipo censo 4 corresponde al ambito local

    INSERT INTO CENSO (cod_censo,tipo_censo) VALUES (codigo,4);
    

  END INSERT_CENSO_LOCAL;
/




--********************************************************************************************
--                         TABLA OPCIONES
--********************************************************************************************

CREATE TABLE OPCIONES(
  cod_opcion NUMBER(5),
  tipo_opcion VARCHAR2(20) NOT NULL,
  CONSTRAINT pk_opciones PRIMARY KEY (cod_opcion),
  CONSTRAINT ck_opcion CHECK (LOWER(tipo_opcion) in ('blanco','nulo','partido politico'))
  );


/
CREATE SEQUENCE SEQ_OPCIONES
  INCREMENT BY 1
  START WITH 1
  NOCYCLE
  NOCACHE;
/
--********************************************************************************************
--                         TABLA OPCIONES_VOTACION
--********************************************************************************************

CREATE TABLE OPCIONES_VOTACION(
  cod_votacion INTEGER,
  cod_opcion NUMBER (5),
  votos_opcion INTEGER,
  porcent_participacion NUMBER(5,2),
  CONSTRAINT pk_opc_votacion PRIMARY KEY (cod_votacion,cod_opcion),
  CONSTRAINT fk_opcion FOREIGN KEY (cod_votacion) REFERENCES VOTACION(cod_votacion) ON DELETE CASCADE,
  CONSTRAINT fk_opcion2 FOREIGN KEY (cod_opcion) REFERENCES OPCIONES (cod_opcion)
  );


--********************************************************************************************
--                         TABLA VOTO_BLANCO
--********************************************************************************************

CREATE TABLE VOTO_BLANCO(
  cod_vblanco NUMBER(1),
  descripcion VARCHAR2(250),
  CONSTRAINT uk_vblanco UNIQUE (descripcion), 
  CONSTRAINT pk_vblanco PRIMARY KEY (cod_vblanco),
  CONSTRAINT fk_vblanco FOREIGN KEY (cod_vblanco) REFERENCES OPCIONES(cod_opcion) ON DELETE CASCADE
  );


/

CREATE OR REPLACE
 TRIGGER INSERT_VOTO_BLANCO
 BEFORE INSERT ON VOTO_BLANCO
 FOR EACH ROW
  
  DECLARE
  
  codigo integer;
  
  BEGIN 
    Select SEQ_OPCIONES.NEXTVAL INTO codigo FROM DUAL;
    :NEW.cod_vblanco:=codigo;
   
-- Tipo opcion 1 corresponde al voto blanco

    INSERT INTO OPCIONES (cod_opcion,tipo_opcion) VALUES (codigo,'blanco');
    

  END INSERT_VOTO_BLANCO;
/



--********************************************************************************************
--                         TABLA VOTO_NULO
--********************************************************************************************  
CREATE TABLE VOTO_NULO(
  cod_vnulo NUMBER(1),
  descripcion VARCHAR2(250),
  CONSTRAINT uk_vnulo UNIQUE (descripcion),
  CONSTRAINT pk_vnulo PRIMARY KEY (cod_vnulo),
  CONSTRAINT fk_vnulo FOREIGN KEY (cod_vnulo) REFERENCES OPCIONES(cod_opcion) ON DELETE CASCADE
  );

/

CREATE OR REPLACE
 TRIGGER INSERT_VOTO_NULO
 BEFORE INSERT ON VOTO_NULO
 FOR EACH ROW
  
  DECLARE
  
  codigo integer;
  
  BEGIN 
    Select SEQ_OPCIONES.NEXTVAL INTO codigo FROM DUAL;
    :NEW.cod_vnulo:=codigo;
   
-- Tipo opcion 2 corresponde al voto nulo

    INSERT INTO OPCIONES (cod_opcion,tipo_opcion) VALUES (codigo,'nulo');
    

  END INSERT_VOTO_NULO;
/


--********************************************************************************************
--                         TABLA PART_POLITICO
--********************************************************************************************

CREATE TABLE PART_POLITICO(
  cod_partido INTEGER,
  siglas VARCHAR2(50),
  nombre VARCHAR2 (50) NOT NULL,
  sede VARCHAR2(50) NOT NULL,
  CONSTRAINT pk_partido PRIMARY KEY (cod_partido),
  CONSTRAINT uk_partido UNIQUE (nombre,siglas),
  CONSTRAINT fk_partido FOREIGN KEY (cod_partido) REFERENCES OPCIONES(cod_opcion) ON DELETE CASCADE
  );

  
/

CREATE OR REPLACE
 TRIGGER INSERT_PART_POLITICO
 BEFORE INSERT ON PART_POLITICO
 FOR EACH ROW
  
  DECLARE
  
  codigo integer;
  
  BEGIN 
    Select SEQ_OPCIONES.NEXTVAL INTO codigo FROM DUAL;
    :NEW.cod_partido:=codigo;
   
-- Tipo opcion 3 corresponde a Partido POlitico

    INSERT INTO OPCIONES (cod_opcion,tipo_opcion) VALUES (codigo,'Partido Politico');
    

  END INSERT_PART_POLITICO;
/
--********************************************************************************************
--                         TABLA CENTRO_ELECTORAL
--********************************************************************************************

CREATE TABLE CENTRO_ELECTORAL(
  cod_centro NUMBER(4),
  cod_clocal INTEGER NOT NULL,
  nombre VARCHAR2(50) NOT NULL,
  direccion VARCHAR2(50),
  CONSTRAINT pk_centro PRIMARY KEY (cod_centro),
  CONSTRAINT uk_centro UNIQUE (cod_clocal,nombre),
  CONSTRAINT fk_centro FOREIGN KEY (cod_clocal) REFERENCES CENSO_LOCAL(cod_clocal) ON DELETE CASCADE
);


/ 
CREATE SEQUENCE SEQ_CENTROELECTORAL
  INCREMENT BY 1
  START WITH 1
  NOCYCLE
  NOCACHE;
/

CREATE OR REPLACE
 TRIGGER INSERT_CENTROELECTORAL
 BEFORE INSERT ON CENTRO_ELECTORAL
 FOR EACH ROW
  BEGIN 
    Select SEQ_CENTROELECTORAL.NEXTVAL INTO :NEW.cod_centro FROM DUAL;
  END INSERT_CENTROELECTORAL;

/

--********************************************************************************************
--                         TABLA MESA
--********************************************************************************************

CREATE TABLE MESA(
  cod_mesa VARCHAR2(50),
  cod_centro NUMBER(4),
  CONSTRAINT pk_mesa PRIMARY KEY (cod_centro,cod_mesa),
  CONSTRAINT fk_mesa FOREIGN KEY (cod_centro) REFERENCES CENTRO_ELECTORAL(cod_centro) ON DELETE CASCADE
);


--********************************************************************************************
--                         TABLA ELECTOR
--********************************************************************************************

CREATE TABLE ELECTOR(
  cod_elector VARCHAR2(15),
  cod_cregional INTEGER NOT NULL,
  cod_clocal INTEGER NOT NULL,
  fecha_nac DATE NOT NULL,
  cod_centro NUMBER(4) NOT NULL,
  cod_mesa VARCHAR2(50) NOT NULL,
  CONSTRAINT pk_elector PRIMARY KEY (cod_elector),
  CONSTRAINT fk_elector1 FOREIGN KEY (cod_cregional) REFERENCES CENSO_REGIONAL(cod_cregional),
  CONSTRAINT fk_elector2 FOREIGN KEY (cod_clocal) REFERENCES CENSO_LOCAL(cod_clocal),
  CONSTRAINT fk_elector3 FOREIGN KEY (cod_centro,cod_mesa) REFERENCES MESA(cod_centro,cod_mesa)
);




--********************************************************************************************
--                         TABLA CANDIDATO
--********************************************************************************************

CREATE TABLE CANDIDATO(
  cod_candidato VARCHAR2(15),
  nombre VARCHAR2(50) NOT NULL,
  apellido1 VARCHAR2(50) NOT NULL,
  apellido2 VARCHAR2(50) NOT NULL, 
  CONSTRAINT pk_candidato PRIMARY KEY (cod_candidato)
);


--********************************************************************************************
--                         TABLA CANDIDATOS_VOTACION_PARTIDO
--********************************************************************************************

CREATE TABLE CANDIDATOS_VOTACION_PARTIDO(
  cod_candidato VARCHAR2(15),
  cod_votacion INTEGER,
  cod_partido INTEGER NOT NULL,
  orden_lista NUMBER(2) NOT NULL,
  CONSTRAINT pk_cand_vot_part PRIMARY KEY (cod_candidato, cod_votacion),
  CONSTRAINT uk_cvp UNIQUE (cod_votacion,cod_partido,orden_lista),
  CONSTRAINT fk_1 FOREIGN KEY (cod_candidato) REFERENCES CANDIDATO (cod_candidato),
  CONSTRAINT fk_2 FOREIGN KEY (cod_votacion) REFERENCES VOTACION (cod_votacion) ON DELETE CASCADE,
  CONSTRAINT fk_3 FOREIGN KEY (cod_partido) REFERENCES PART_POLITICO (cod_partido)
);


--********************************************************************************************
--                         TABLA VOTAR
--********************************************************************************************

CREATE TABLE VOTAR(
  cod_votante VARCHAR2(15),
  cod_votacion INTEGER,
  cod_opcion NUMBER(5) NOT NULL,
  CONSTRAINT pk_votar PRIMARY KEY (cod_votante, cod_votacion),
  CONSTRAINT fk_4 FOREIGN KEY (cod_votante) REFERENCES ELECTOR (cod_elector),
  CONSTRAINT fk_5 FOREIGN KEY (cod_votacion) REFERENCES VOTACION (cod_votacion),
  CONSTRAINT fk_6 FOREIGN KEY (cod_opcion) REFERENCES OPCIONES (cod_opcion)
);


--********************************************************************************************
--                         TABLA EST_VOT_FINAL_AO
--********************************************************************************************

CREATE TABLE EST_VOT_FINAL_AO(
 cod_pais VARCHAR2(2),
 anyo NUMBER(4),
 vot_finalizadas NUMBER(4) NOT NULL,
 CONSTRAINT pk3 PRIMARY KEY (cod_pais,anyo),
 CONSTRAINT fk3 FOREIGN KEY (cod_pais) REFERENCES CENSO_NACIONAL(cod_pais)
);


--********************************************************************************************
--                         TABLA EST_CENSO_ANYO_PORCT
--********************************************************************************************

CREATE TABLE EST_CENSO_ANYO_PORCT(
 cod_censo INTEGER,
 anyo NUMBER(4),
 porct_medio_participacion NUMBER(5,2) NOT NULL,
 CONSTRAINT pk2 PRIMARY KEY (cod_censo,anyo),
 CONSTRAINT fk2 FOREIGN KEY (cod_censo) REFERENCES CENSO ON DELETE CASCADE
);

--********************************************************************************************
--                         TABLA EST_CENSO_ANYO_VOTA
--********************************************************************************************

CREATE TABLE EST_CENSO_ANYO_VOTA(
 cod_censo INTEGER,
 anyo NUMBER(4),
 num_votacion NUMBER(4) NOT NULL,
 CONSTRAINT pk10 PRIMARY KEY (cod_censo,anyo),
 CONSTRAINT fk10 FOREIGN KEY (cod_censo) REFERENCES CENSO ON DELETE CASCADE
);


--********************************************************************************************
--                         TABLA EST_VOT_MAS_PARTI_ANYO
--********************************************************************************************

CREATE TABLE EST_VOT_MAS_PARTI_ANYO(
 cod_votacion INTEGER,
 anyo NUMBER(4) NOT NULL,
 participacion NUMBER(5,2) NOT NULL,
 CONSTRAINT pk4 PRIMARY KEY (cod_votacion),
 CONSTRAINT fk4 FOREIGN KEY (cod_votacion) REFERENCES VOTACION(cod_votacion)
);


--********************************************************************************************
--                         TABLA EST_LOC_MAS_VOTOS_ANYO
--********************************************************************************************

CREATE TABLE EST_LOC_MAS_VOTOS_ANYO(
 cod_localidad VARCHAR2(10),
 nom_localidad VARCHAR(50),
 anyo NUMBER(4),
 votos INTEGER NOT NULL,
 CONSTRAINT pk5 PRIMARY KEY (cod_localidad,anyo)
);


--********************************************************************************************
--                         TABLA EST_CIUDADANOS_SINVOTOS_ANYO
--********************************************************************************************
CREATE TABLE EST_CIUDADANOS_SINVOTOS_ANYO(
 anyo NUMBER(4),
 porct_sinvotar NUMBER(5,2) NOT NULL,
 CONSTRAINT pk6 PRIMARY KEY (anyo)
 );
 
 
--********************************************************************************************
--                         TABLA EST_VOTAC_PAIS_PERSONA
--********************************************************************************************
 CREATE TABLE EST_VOTAC_PAIS_PERSONA(
 cod_elector VARCHAR2(25),
 cod_pais VARCHAR2(2) NOT NULL,
 votaciones NUMBER(2) NOT NULL,
 CONSTRAINT pk7 PRIMARY KEY (cod_elector),
 CONSTRAINT fk7 FOREIGN KEY (cod_elector) REFERENCES ELECTOR(cod_elector),
 CONSTRAINT fk8 FOREIGN KEY (cod_pais) REFERENCES CENSO_NACIONAL (cod_pais)
);


--********************************************************************************************
--                         TABLA EST_HISTORICO_VOTACIONES
--********************************************************************************************
CREATE TABLE EST_HISTORICO_VOTACIONES(
 cod_votacion INTEGER,
 porct_participacion NUMBER(5,2) NOT NULL,
 CONSTRAINT pk9 PRIMARY KEY (cod_votacion),
 CONSTRAINT fk9 FOREIGN KEY (cod_votacion) REFERENCES VOTACION(cod_votacion)
);

--********************************************************************************************
--                         TABLA EST_HISTORICO_MAS
--********************************************************************************************
CREATE TABLE EST_HISTORICO_MAS(
 cod_votacion INTEGER,
 porct_masvotada NUMBER(5,2) NOT NULL,
 CONSTRAINT pk11 PRIMARY KEY (cod_votacion),
 CONSTRAINT fk11 FOREIGN KEY (cod_votacion) REFERENCES VOTACION(cod_votacion)
);

--********************************************************************************************
--                         TABLA EST_HISTORICO_MENOS
--********************************************************************************************
CREATE TABLE EST_HISTORICO_MENOS(
 cod_votacion INTEGER,
 porct_menosvotada NUMBER(5,2) NOT NULL,
 CONSTRAINT pk12 PRIMARY KEY (cod_votacion),
 CONSTRAINT fk12 FOREIGN KEY (cod_votacion) REFERENCES VOTACION(cod_votacion)
);
--********************************************************************************************
--                         TABLA LOGS
--********************************************************************************************

CREATE TABLE LOGS(
 cod_log INTEGER,
 nom VARCHAR2(250)NOT NULL,
 fecha DATE NOT NULL,
 param_entrada VARCHAR2(255),
 param_salida VARCHAR2(255),
 CONSTRAINT PK_LOGS PRIMARY KEY(cod_log)
);


/
CREATE SEQUENCE SEQ_LOGS
  INCREMENT BY 1
  START WITH 1
  NOCYCLE
  NOCACHE;
/
CREATE OR REPLACE
 TRIGGER INSERT_LOGS
 BEFORE INSERT ON LOGS
 FOR EACH ROW
  BEGIN 
    Select SEQ_LOGS.NEXTVAL INTO :NEW.cod_log FROM DUAL;
  END INSERT_LOGS;
/
COMMIT;


--********************************************************************************************
--                         TABLA ELECTORES
--********************************************************************************************

CREATE TABLE ELECTORES(
  cod_votacion integer
);


--********************************************************************************************
--                         VISTA PAISCENSO
--********************************************************************************************

CREATE OR REPLACE  VIEW PAISCENSO AS 
select cod_cnacional codcenso,cod_pais from censo_nacional 
union
select cod_cregional,cod_pais from censo_regional 
union
select cod_clocal,cod_pais from censo_local;



--********************************************************************************************
--                         VISTA VOTACENSO
--********************************************************************************************

 CREATE OR REPLACE  VIEW VOTACENSO AS 
 select v.cod_votacion,ch.cod_censo, v.fecha_inicio,v.porcent_participacion, v.estado from votacion v, censos_habilitados ch where v.cod_votacion=ch.cod_votacion;


--********************************************************************************************
--                         VISTA VOTALOCAL
--********************************************************************************************

CREATE OR REPLACE VIEW  VOTALOCAL as
select  cl.cod_localidad,cl.nom_localidad, extract(year from fecha_inicio) anyo,count(cod_localidad) totalvotos from votacion v, censos_habilitados ch,censo c, votar vt, elector e, censo_local cl 
where v.cod_votacion=ch.cod_votacion and ch.cod_censo=c.cod_censo and v.cod_votacion=vt.cod_votacion and c.tipo_censo=4 and vt.cod_votante=e.cod_elector and ch.cod_censo=e.cod_clocal and e.cod_clocal=cl.cod_clocal
group by extract(year from fecha_inicio), cod_localidad, nom_localidad;

--********************************************************************************************
--                         VISTA CENSOSELECTOR
--********************************************************************************************

CREATE OR REPLACE VIEW CENSOSELECTOR  AS 
  select  e.cod_elector, e.cod_cregional,e.cod_clocal,cn.cod_cnacional,cn.cod_pais from elector e , censo_regional cr, censo_nacional cn   where e.cod_cregional= cr.cod_cregional and cr.cod_pais=cn.cod_pais;


--********************************************************************************************
--                         VISTA MAXVOTACION
--********************************************************************************************

CREATE OR REPLACE VIEW MAXVOTACION AS
select * from (
select cod_votante votante, count(cod_votacion) tot from votar group by cod_votante)
where tot=(select  max(total) from (
select count(vt.cod_votacion) total from votar vt group by vt.cod_votante));

 